#! /usr/bin/env python
# this is an enhanced list of bifurcation branches:
# the labels contain the solutions and diagnostics as well
from auto import AUTOutil
from auto import parseB
from auto import parseC
from auto import parseH
from auto import parseS
from auto import parseD
from auto import Points
from auto import AUTOExceptions
import gzip
import types
import sys
import struct

class bifDiag(parseB.parseBR):

    # some constants must not be preserved from run to run. These are:
    nonekeys = set(["IRS", "PAR", "U", "sv", "s", "dat", "TY", "LAB", "IBR"])

    def __init__(self,fort7_filename=None,fort8_filename=None,
                 fort9_filename=None,constants=None):
        if (constants is not None and constants.get('sv') is not None and
            type(fort7_filename) in (type(""),type(None)) and
            type(fort8_filename) in (type(""),type(None)) and
            type(fort9_filename) in (type(""),type(None))):
            #filebased
            self.c = constants
            self.filenames = [fort7_filename,fort8_filename,fort9_filename]
        else:
            self.__realinit(fort7_filename,fort8_filename,fort9_filename,
                            constants)

    def __realinit(self,fort7_filename,fort8_filename,fort9_filename,constants):
        ioerrors = []
        try:
            parseB.parseBR.__init__(self,fort7_filename)
            for i, d in enumerate(self):
                self[i] = bifDiagBranch(d)
        except IOError:
            ioerrors.append(str(sys.exc_info()[1]))
            parseB.parseBR.__init__(self)
            fort7_filename = None
        if isinstance(fort8_filename, parseS.AUTOSolution):
            fort8_filename = [fort8_filename]
        try:
            solution = parseS.parseS(fort8_filename)
        except IOError:
            ioerrors.append(str(sys.exc_info()[1]))
            solution = None
            if fort7_filename is None:
                raise AUTOExceptions.AUTORuntimeError('\n'.join(ioerrors))
        if fort7_filename is None and fort8_filename is not None:
            # simulate a bifurcation diagram
            labels = {}
            for s in solution:
                br = s["Branch number"]
                if labels == {} or br != branch.BR:
                    if labels != {}:
                        branch.labels = Points.PointInfo(labels)
                    branch = bifDiagBranch()
                    self.append(branch)
                    branch.BR = br
                    branch.coordarray = []
                    branch.coordnames = []
                    branch.headernames = []
                    branch.headerlist = []
                    branch.c = constants
                    labels = {}
                    base = 0
                    i = 0
                pt = s["PT"]
                lab = s["LAB"]
                ty = s["TY number"]
                if i >= base + pt - 1:
                    # point numbers wrapped around
                    base += 9999
                i = base + pt - 1
                labels[i] = {s["TY"]: {"LAB":lab,"TY number":ty,"PT":pt}}
            if labels != {}:
                branch.labels = Points.PointInfo(labels)
        if fort9_filename is not None and self.data != []:
            try:
                # for now just attach diagnostics information to first branch
                self[0].diagnostics = parseD.parseD(fort9_filename)
            except IOError:
                pass
        i = 0
        if solution is not None:
            for d in self:
                if constants is None and d.c is not None:
                    constants = d.c
                constants = parseC.parseC(constants)
                for k in constants:
                    if k in self.nonekeys:
                        constants[k] = None
                for ind in d.labels.getIndices():
                    if i >= len(solution):
                        break
                    x = d._gettypelabel(ind)[1]
                    s = solution[i]
                    if x.get("LAB",0) != 0 or s["LAB"] == 0:
                        i = i+1
                        s = x["solution"] = parseS.AUTOSolution(
                            s, constants=constants)
                        if d.coordnames != []:
                            s.b = d[ind]

    #delayed file-based reading to save memory if sv= is used in run()
    def __getattr__(self,attr):
        if attr == 'c':
            if len(self)>0 and self[0].c is not None:
                return self[0].c
        elif attr == 'data':
            if hasattr(self,'filenames'):
                self.__realinit(self.filenames[0], self.filenames[1],
                                self.filenames[2], self.c)
                del self.filenames
                del self.c
                return self.data
        return super(bifDiag, self).__getattribute__(attr)
        
    def __repr__(self):
        result = id(self)
        if result < 0:
            # avoid negative addresses and Python 2.3 warnings
            result += 256 ** struct.calcsize('P')
        return "<_=%s instance at %#010x>"%(self.__class__.__name__,result)

    def getLabel(self,label):
        sols = parseS.parseS()
        #adjust maximum label/branch
        mbr = max([abs(d["BR"]) for d in self] or [None])
        mlab = max(self.getLabels() or [None])
        for d in self:
            sols.extend(d.getLabel(None,mbr=mbr,mlab=mlab))
        return sols(label)

    def __call__(self,label=None):
        return self.getLabel(label)

    def load(self,**kw):
        """Load solution with the given AUTO constants.
        Returns a shallow copy with a copied set of updated constants
        """
        return self().load(**kw)

    def read(self,fort7_input,fort8_input=None,fort9_input=None):
        parseB.parseBR.read(self,fort7_input)
        for i, d in enumerate(self):
            self[i] = bifDiagBranch(d)
        if fort8_input is not None and (
            isinstance(fort8_input, (file, gzip.GzipFile))):
            solution = parseS.parseS()
            solution.read(fort8_input)
            i = 0
            for d in self:
                for k,x in map(d._gettypelabel, d.labels.getIndices()):
                    if x.get("LAB",0) != 0:
                        x["solution"] = solution[i]
                        i = i+1
        if fort9_input is not None:
            diagnostics = parseD.parseD()
            diagnostics.read(fort9_input)
            # for now just attach diagnostics information to the first branch
            self[0].diagnostics = diagnostics

    def write(self,fort7_output,fort8_output=None,fort9_output=None):
        parseB.parseBR.write(self,fort7_output)
        if fort8_output is not None:
            self().write(fort8_output)
        if fort9_output is not None:
            for d in self:
                if hasattr(d,"diagnostics"):
                    d.diagnostics.write(fort9_output)

    def readFilename(self,fort7_filename,fort8_filename=None,fort9_filename=None):
        parseB.parseBR.readFilename(self,fort7_filename)
        for i, d in enumerate(self):
            self[i] = bifDiagBranch(d)
        if fort8_filename is not None and isinstance(fort8_filename, str):
            solution = parseS.parseS(fort8_filename)
            i = 0
            for d in self:
                for k,x in map(d._gettypelabel, d.labels.getIndices()):
                    if x.get("LAB",0) != 0:
                        x["solution"] = solution[i]
                        i = i+1
        if not fort9_filename is None:
            # for now just attach diagnostics information to the first branch
            self[0].diagnostics = parseD.parseD(fort9_filename)

    def writeFilename(self,fort7_filename,fort8_filename=None,fort9_filename=None,append=False):
        #if only one filename is given, then just save the solutions file
        if fort8_filename is None:
            fort8_filename = fort7_filename
        elif len(self) > 0 and len(self[0]) > 0:
            parseB.parseBR.writeFilename(self,fort7_filename,append)
        if fort8_filename != '':
            self().writeFilename(fort8_filename,append)
        if not fort9_filename is None:
            for d in self:
                if hasattr(d,"diagnostics"):
                    d.diagnostics.writeFilename(fort9_filename,append)
                    append=True

    # Removes solutions with the given labels or type names
    def deleteLabel(self,label=None,keepTY=0,keep=0,copy=0):
        # accept a user-defined boolean function
        if isinstance(label, types.FunctionType):
            deletesols = self(label)
            data = []
            for d in self:
                label = [s["LAB"] for s in deletesols if s.b.branch is d]
                data.append(d.deleteLabel(label,keepTY,keep,copy))
            if copy:
                return self.__class__(data)
            return
        return parseB.parseBR.deleteLabel(self,label,keepTY,keep,copy)

class bifDiagBranch(parseB.AUTOBranch):
    def __init__(self,input=None):
        parseB.AUTOBranch.__init__(self,input)

    def __getattr__(self,attr):
        if attr == "diagnostics":
            raise AttributeError
        return super(bifDiagBranch, self).__getattr__(attr)

    def getLabel(self,label,mbr=None,mlab=None):
        sols = []
        for idx in self.labels.getIndices():
            x = self._gettypelabel(idx)[1]
            if "solution" in x:
                br = abs(self["BR"])
                pt = idx%9999 + 1
                lab = x["LAB"]
                ty = x["TY number"]
                sol = x["solution"]
                if (sol._mlab != mlab or sol._mbr != mbr or
                    br != sol["BR"] or pt != sol["PT"] or
                    ty != sol["TY number"] or lab != sol["LAB"]):
                    sol = sol.__class__(sol, BR=br, PT=pt,
                                        LAB=lab, TY=ty)
                    sol._mlab = mlab
                    sol._mbr = mbr
                sols.append(sol)
        return parseS.parseS(sols)(label)

    def load(self,**kw):
        """Load solution with the given AUTO constants.
        Returns a shallow copy with a copied set of updated constants
        """
        return self().load(**kw)

    def write(self,fort7_output,fort8_output=None,fort9_output=None):
        parseB.AUTOBranch.write(self,fort7_output)
        if fort8_output is not None:
            self().write(fort8_output)
        if fort9_output is not None:
            if hasattr(self,"diagnostics"):
                self.diagnostics.write(fort9_output)

    def writeFilename(self,fort7_filename,fort8_filename=None,fort9_filename=None,append=False):
        #if only one filename is given, then just save the solutions file
        if fort8_filename is None:
            fort8_filename = fort7_filename
        elif len(self) > 0:
            parseB.AUTOBranch.writeFilename(self,fort7_filename,append)
        if fort8_filename != '':
            self().writeFilename(fort8_filename,append)
        if not fort9_filename is None:
            if hasattr(self,"diagnostics"):
                self.diagnostics.writeFilename(fort9_filename,append)
                append=True

    def deleteLabel(self,label=None,keepTY=0,keep=0,copy=0):

        # accept a user-defined boolean function
        if isinstance(label, types.FunctionType):
            label = [s["LAB"] for s in self(label)]
        new = parseB.AUTOBranch.deleteLabel(self,label,keepTY,keep,copy)
        if keepTY:
            if not copy:
                new = self
            for idx in new.labels.getIndices():
                x = new._gettypelabel(idx)[1]
                if x["LAB"] ==0 and "solution" in x:
                    del x["solution"]
        if copy:
            if hasattr(self,"diagnostics"):
                new.diagnostics = self.diagnostics
            return new

    def relabel(self,old_label=1,new_label=None):
        """Relabels the first solution with the given label"""
        if new_label is None:
            new = self.__class__(parseB.AUTOBranch.relabel(self,old_label))
            if hasattr(self,"diagnostics"):
                new.diagnostics = self.diagnostics
            return new
        parseB.AUTOBranch.relabel(self,old_label,new_label)
    
def pointtest7(a,b):
    if "TY name" not in a:
        raise AUTOExceptions.AUTORegressionError("No TY name label")
    if "TY number" not in a:
        raise AUTOExceptions.AUTORegressionError("No TY number label")
    if "BR" not in a:
        raise AUTOExceptions.AUTORegressionError("No BR label")
    if "data" not in a:
        raise AUTOExceptions.AUTORegressionError("No data label")
    if "PT" not in a:
        raise AUTOExceptions.AUTORegressionError("No PT label")
    if "LAB" not in a:
        raise AUTOExceptions.AUTORegressionError("No LAB label")
    if len(a["data"]) != len(b["data"]):
        raise AUTOExceptions.AUTORegressionError("Data sections have different lengths")
   
    
def pointtest8(a,b):
    keys = ['Type number', 'Type name',
            'Free Parameters',  'Branch number', 'Parameter NULL vector',
            'data', 'NCOL', 'Label', 'ISW', 'NTST',
            'Point number', 'Parameters']

    scratch=a['Parameters']
    scratch=b['Parameters']
    for key in keys:
        if key not in a:
            raise AUTOExceptions.AUTORegressionError("No %s label"%(key,))
    if len(a["data"]) != len(b["data"]):
        raise AUTOExceptions.AUTORegressionError("Data sections have different lengths")


def test():
    foo = bifDiag()
    foo.readFilename("test_data/fort.7","test_data/fort.8")
    if len(foo[0]) != 150:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest7(foo[0].getIndex(0),foo[0].getIndex(57))
    if len(foo()) != 5:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest8(foo().getIndex(0),foo().getIndex(3))

    if len(foo.getLabels()) != 5:
        raise AUTOExceptions.AUTORegressionError("Incorrect number of labels")

    print("Deleting labels")
    foo.deleteLabel(range(6,9))
    
    if len(foo.getLabels()) != 2:
        raise AUTOExceptions.AUTORegressionError("Incorrect number of labels")

    print("Relabeling")
    foo.relabel(9,57)

    for i in range(len(foo[0])):
        if foo[0].getIndex(0)["TY number"] != 0:
            if foo[0].getIndex(0)["LAB"] != 57:
                raise AUTOExceptions.AUTORegressionError("Incorrect label")
            break
    if foo().getIndex(0)["Label"] != 57:
        raise AUTOExceptions.AUTORegressionError("Incorrect label")

    print("Making labels unique")
    foo.uniquelyLabel()

    for i in range(len(foo[0])):
        if foo[0].getIndex(0)["TY number"] != 0:
            if foo[0].getIndex(0)["LAB"] != 1:
                raise AUTOExceptions.AUTORegressionError("Incorrect label")
            break
    if foo().getIndex(0)["Label"] != 1:
        raise AUTOExceptions.AUTORegressionError("Incorrect label")

    print("bifDiag passed all tests")

if __name__ == '__main__' :
    test()



#! /usr/bin/env python
#    Visualization for Bifurcation Manifolds
#    Copyright (C) 1997 Randy Paffenroth and John Maddocks
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU  General Public
#    License as published by the Free Software Foundation; either
#    version 2 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this library; if not, write to the Free
#    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
#    MA 02111-1307, USA

import os
import sys
import struct
try:
    from UserDict import UserDict
    from UserList import UserList
except ImportError: #Python 3
    from collections import UserDict, UserList
try:
    file
except NameError: #Python 3
    from io import IOBase as file
from auto import AUTOExceptions
import copy
from auto import parseB
from auto import parseC
from auto import Points
import gzip
from auto import AUTOutil
from auto.AUTOutil import format19_10E3
import types

# End of data exception definition
class PrematureEndofData(Exception):
    pass

class IncorrectHeaderLength(Exception):
    pass

# This is the number of parameters.  Should be read from auto.h...
# This is not required anymore in AUTO97, since the last entry in
# the header line is this number
NPAR = 20

class fileS(object):
    def __init__(self, filename):
        if isinstance(filename, str):
            inputfile = AUTOutil.openFilename(filename,"rb")
        else:
            inputfile = filename
        self.inputfile = inputfile
        self.name = inputfile.name
        self.solutions = []

        # We now go through the file and read the solutions.
        prev = None
        # for fort.8 we need to read everything into memory; otherwise load the
        # data on demand from disk when we really need it
        # on Windows always load everything because deleting open files is
        # impossible there
        inmemory = (os.path.basename(inputfile.name) == 'fort.8' or
                    sys.platform in ['cygwin', 'win32'])
        while len(inputfile.read(1)) > 0:
            line = inputfile.readline()
            if not line: raise PrematureEndofData
            try:
                header = list(map(int, line.split()))
            except ValueError:
                raise PrematureEndofData
            if len(header) < 10:
                raise PrematureEndofData
            if len(header) == 10:
                # This is the case for AUTO94 and before
                header = header + [NPAR]
            numLinesPerEntry = header[8]
            start_of_data = inputfile.tell()
            if prev is not None and all(
                [header[i] == prevheader[i] for i in [4, 6, 7, 8, 11]]):
                # guess the end from the previous solution
                end += inputfile.tell() - prev
                # See if the guess for the solution end is correct
                inputfile.seek(end)
                data = inputfile.readline().split()
                # This is where we detect the end of the file
                if len(data) == 0:
                    data = inputfile.read(1)
                if len(data) != 0:
                    try:
                        # Check length of line...
                        if len(data) != 12 and len(data) != 16:
                            raise IncorrectHeaderLength
                        # and the fact they are all integers
                        map(int,data)
                        # and the fact that NCOL*NTST+1=NTPL
                        if int(data[9])*int(data[10])+1 != int(data[6]):
                            end = None
                        # If it passes all these tests we say it is a header line
                        # and we can read quickly
                    except:
                        # otherwise the guessed end is not valid
                        end = None
            else:
                end = None
            data = None
            if end is None:
                # We skip the correct number of lines in the entry to
                # determine its end.
                inputfile.seek(start_of_data)
                if inmemory:
                    data = "".encode("ascii").join([inputfile.readline()
                                   for i in range(numLinesPerEntry)])
                else:
                    for i in range(numLinesPerEntry):
                        inputfile.readline()
                end = inputfile.tell()
            elif inmemory:
                inputfile.seek(start_of_data)
                data = inputfile.read(end - start_of_data)
            else:
                inputfile.seek(end)
            if data is None:
                data = (start_of_data, end)
            self.solutions.append({'header': header, 'data': data})
            prev = start_of_data
            prevheader = header

    def readstr(self, i):
        solution = self.solutions[i]
        data = solution['data']
        if not isinstance(data, tuple):
            return data
        start = data[0]
        end = data[1]
        self.inputfile.seek(start)
        solution['data'] = self.inputfile.read(end - start)
        return solution['data']

    def readfloats(self, i, total):
        if not Points.numpyimported:
            Points.importnumpy()       
        N = Points.N
        data = self.readstr(i)
        if hasattr(N, "ndarray") and isinstance(data, N.ndarray):
            return data
        fromstring = Points.fromstring
        if fromstring:
            fdata = []
            if "D".encode("ascii") not in data:
                fdata = fromstring(data, dtype=float, sep=' ')
            if fdata == [] or len(fdata) != total:
                fdata = N.array(map(parseB.AUTOatof,
                                    data.split()), 'd')
            else:
                #make sure the last element is correct
                #(fromstring may not do this correctly for a
                #string like -2.05071-106)
                fdata[-1] = parseB.AUTOatof(
                    data[data.rfind(" ".encode("ascii"))+1:].strip())
        else:
            data = data.split()
            try:
                fdata = N.array(map(float, data), 'd')
            except ValueError:
                fdata = N.array(map(parseB.AUTOatof, data), 'd')
        if total != len(fdata):
            raise PrematureEndofData
        del self.solutions[i]['data']
        self.solutions[i]['data'] = fdata
        return fdata

    def conditionalclose(self):
        # if everything is in memory, close the file
        for s in self.solutions:
            if isinstance(s['data'], tuple):
                return
        self.inputfile.close()

# The parseS class parses an AUTO fort.8 file
# THESE EXPECT THE FILE TO HAVE VERY SPECIFIC FORMAT!
# it provides 4 methods:
# read and write take as an arguement either and input or output
#    stream (basically any object with has the method "readline"
#    for reading and "write" for writing)
#    
# readFilename and writeFilename take as an arguement a filename
#    in which to read/write the parameters (basically it opens the
#    file and then calles "read" or "write"
#    
# Once the data is read in the class provides a list all the points
# in the fort.8 file.

class parseS(list):
    def __init__(self,filename=None):
        if isinstance(filename, str):
            list.__init__(self)
            self.readFilename(filename)
        else:
            if filename is None:
                list.__init__(self)
            else:
                list.__init__(self,filename)

    def __str__(self):
        rep = ""
        rep = rep + "Number of solutions: " + str(len(self)) + "\n"
        labels = self.getLabels()
        rep = rep + "Labels: "
        for label in labels:
            rep = rep + str(label) + " "
        rep = rep + "\n"
        return rep

    def __call__(self,label=None):
        return self.getLabel(label)

    def load(self,**kw):
        """Load solution with the given AUTO constants.
        Returns a shallow copy with a copied set of updated constants
        """
        irs = kw.get("IRS")
        if irs is None:
            irs = (kw.get("constants") or {}).get("IRS")
        sol = None
        if irs is not None:
            if irs != 0:
                try:    
                    sol = self(irs)
                except KeyError:
                    pass
        elif len(self) > 0:
            sol = self[-1]
        else:
            raise AUTOExceptions.AUTORuntimeError(
                "Cannot start from empty solution list.")
        if sol is None:
            sol = AUTOSolution()
        return sol.load(**kw)

    # This function needs a little explanation
    # It trys to read a new point from the input file, and if
    # it cannot (because the file ends prematurely) is sets the
    # file pointer back to the way it was when it started and returns
    # This will be used to check a file to see if it has a new point
    # and it will ignore a partially created solution.
    # Basically it for an VBM kind of program.
    def tryNextPointRead(self,inputfile):
        current_position = inputfile.tell()
        try:
            self.append(AUTOSolution(inputfile,name=inputfile.name))
        except PrematureEndofData:
            inputfile.seek(current_position)

    def read(self,inputfile=None):
        # We now go through the file and read the solutions.
        if inputfile is None:
            # read everything into memory
            for solution in self:
                solution.read()
            return
        if not isinstance(inputfile, fileS):
            inputfile = fileS(inputfile)
        for i in range(len(inputfile.solutions)):
            solution = AUTOSolution(inputfile,i,inputfile.name)
            self.append(solution)
        if len(self) > 0:
            mbr, mlab = 0, 0
            for d in self:
                if d["BR"] > mbr: mbr = d["BR"]
                if d["LAB"] > mlab: mlab = d["LAB"]
            for d in self:
                d._mbr, d._mlab = mbr, mlab

    def write(self,output,mlab=False):
        for i in range(len(self)):
            x = self[i]
            if i == len(self) - 1:
                # maybe write a header after the last solution so that AUTO can
                # pickup a new branch and solution label number
                x.write(output,mlab)
            else:
                x.write(output)
        output.flush()

    def readFilename(self,filename):
        inputfile = fileS(filename)
        self.read(inputfile)
        inputfile.conditionalclose()
        # else don't close but garbage collect

    def writeFilename(self,filename,append=False,mlab=False):
        # read all solutions because we may overwrite
        self.read()
        if append:
            output = open(filename,"ab")
        else:
            output = open(filename,"wb")
        self.write(output,mlab)
        output.close()

    # Removes solutions with the given labels or type name
    def deleteLabel(self,label=None,keep=0):
        if label is None:
            label = [lab for lab in parseB.all_point_types if lab not in
                     ["No Label", "RG", "UZ"]]
        if isinstance(label, (str, int)):
            label = [label]
        indices = []
        for i in range(len(self)):
            x = self[i]
            if ((not keep and (x["Label"] in label or x["Type name"] in label)
                 or (keep and not x["Label"] in label and 
                              not x["Type name"] in label))):
                indices.append(i)
        indices.reverse()
        for i in indices:
            del self[i]
        if len(self) > 0:
            maxlab = max(self.getLabels())
            for d in self:
                d._mlab = maxlab
            
    # Relabels the first solution with the given label
    def relabel(self,old_label=None,new_label=None):
        if old_label is None and new_label is None:
            i = 1
            new = parseS()
            for d in self:
                news = d.__class__(d)
                news["LAB"] = i
                news._mlab = len(self)
                i = i + 1
                new.append(news)
            return new
        if isinstance(old_label, int):
            old_label = [old_label]
            new_label = [new_label]
        for j in range(len(old_label)):
            for d in self:
                if d["Label"] == old_label[j]:
                    d["Label"] = new_label[j]
        if len(self) > 0:
            maxlab = max(self.getLabels())
            for d in self:
                d._mlab = maxlab

    # Make all labels in the file unique and sequential
    def uniquelyLabel(self):
        i = 1
        for d in self:
            d["Label"] = i
            d._mlab = len(self)
            i = i + 1

    # Given a label, return the correct solution
    def getLabel(self,label):
        if label is None:
            return self
        if isinstance(label, int):
            for d in self:
                if d["Label"] == label:
                    return d
            raise KeyError("Label %s not found"%label)
        if isinstance(label, str) and len(label) > 2 and label[-1].isdigit():
            j = 2
            if not label[2].isdigit():
                j = 3
            number = int(label[j:])
            i = 0
            for d in self:
                if d["Type name"] == label[:j]:
                    i = i + 1
                    if i == number:
                        return d
            raise KeyError("Label %s not found"%label)
        if isinstance(label, types.FunctionType):
            # accept a user-defined boolean function
            f = label
            cnt = getattr(f,"func_code",getattr(f,"__code__",None)).co_argcount
            if cnt == 1:
                # function takes just one parameter
                s = [s for s in self if f(s)]
            elif cnt == 2:
                # function takes two parameters: compare all solutions
                # with each other
                indices = set([])
                for i1, s1 in enumerate(self):
                    if i1 in indices:
                        continue
                    for i2 in range(i1+1,len(self)):
                        if i2 not in indices and f(s1, self[i2]):
                            indices.add(i2)
                s = [self[i] for i in sorted(indices)]
            else:
                raise AUTOExceptions.AUTORuntimeError(
                    "Invalid number of arguments for %s."%f.__name__)
            return self.__class__(s)
        if not AUTOutil.isiterable(label):
            label = [label]
        data = []
        counts = [0]*len(label)
        for d in self:
            ap = None
            if d["Label"] in label or d["Type name"] in label:
                ap = d
            for i in range(len(label)):
                lab = label[i]
                j = 2
                if len(lab) > 2 and not lab[2].isdigit():
                    j = 3
                if (isinstance(lab, str) and len(lab) > j and
                    d["Type name"] == lab[:j]):
                    counts[i] = counts[i] + 1
                    if counts[i] == int(lab[j:]):
                        ap = d
            if ap is not None:
                data.append(ap)
        return self.__class__(data)

    def getIndex(self,index):
        return self[index]

    # Return a list of all the labels in the file.
    def getLabels(self):
        return [x["Label"] for x in self]

# an old-style point and point keys within an AUTOSolution
class SLPointKey(UserList):
    def __init__(self, solution=None, index=None, coords=None):
        if coords=="u dot":
            self.solution = solution["udotps"]
        else:
            self.solution = solution
        self.index = index
    def __getattr__(self, attr):
        if attr == 'data':
            return [point[self.index] for point in self.solution.coordarray]
        raise AttributeError(attr)
    def __setitem__(self, i, item):
        self.solution.coordarray[i,self.index] = item
    def __str__(self):
        return str(self.data)
    def append(self, item):
        self.enlarge(1)
        self.solution.coordarray[-1,self.index] = item
        self.data.append(item)
    def extend(self, other):
        self.enlarge(len(other))
        for i in range(len(other)):
            self.solution.coordarray[-len(other)+i,self.index] = other[i]
    def enlarge(self, ext):
        # enlarges the dimension of coordarray and coordnames
        s = self.solution
        if s._dims is None:
            s._dims = [s.dimension]*len(s)
            s0 = s.coordnames[0]
            s.extend([s0[0:s0.find('(')+1]+str(s.dimension+i+1)+')'
                      for i in range(ext)])
        s._dims[self.index] = s.dimension
        if min(s._dims) == max(s._dims):
            s._dims = None

class SLPoint(Points.Point):
    def __init__(self, p, solution=None, index=None):
        Points.Point.__init__(self, p)
        self.index = index
        self.solution = solution

    def __contains__(self, key):
        return key in ["u", "u dot", "t"] or Points.Point.has_key(self,key)

    def has_key(self, key):
        return self.__contains__(key)

    def __getitem__(self, coords):
        if coords == "t":
            return self.solution.indepvararray[self.index]
        if coords in ["u", "u dot"]:
            return SLPointKey(self.solution, self.index, coords)
        return Points.Point.__getitem__(self, coords)

    def __setitem__(self, coords, item):
        if coords == 't':
            self.solution.indepvararray[self.index] = item
        Points.Point.__setitem__(self, coords, item)

    def __str__(self):
        return str({ "t" : self["t"], "u" : self["u"], "u dot" : self["u dot"]})

    __repr__ = __str__

class AUTOParameters(Points.Point):
    def __init__(self, kwd=None, **kw):
        if isinstance(kwd, self.__class__):
            for k,v in kwd.__dict__.items():
                self.__dict__[k] = v
            return
        if kwd is None and kw == {}:
            self.coordnames = []
            self.dimension = 0
            return
        coordnames = kw.get("coordnames",[])[:]
        if "coordarray" in kw:
            coordarray = kw["coordarray"]
            if len(coordarray) < len(coordnames):
                kw["coordarray"] = (list(coordarray) +
                                    (len(coordnames)-len(coordarray)) * [0.0])
            for i in range(len(coordnames),len(coordarray)):
                coordnames.append("PAR(%d)"%(i+1))
            kw["coordtype"] = Points.float64
            kw["coordnames"] = coordnames
        Points.Point.__init__(self,kwd,**kw)

    def __call__(self,index):
        return self.coordarray[index-1]

    def __str__(self):
        rep = []
        for i in range(1,len(self)+1,5):
            j = min(i+4,len(self))
            line = "PAR(%-7s "%("%d:%d):"%(i,j))
            for k in range(i,j+1):
                if self.coordnames[k-1] != "PAR(%d)"%k:
                    for k in range(i,j+1):
                        line += "    %-15s"%self.coordnames[k-1]
                    rep.append(line)
                    line = 12*" "
                    break
            line += "".join([format19_10E3(self(k)) for k in range(i,j+1)])
            rep.append(line)
        return "\n".join(rep)

# The AUTOsolution class parses an AUTO fort.8 file
# THESE EXPECT THE FILE TO HAVE VERY SPECIFIC FORMAT!
# it provides 4 methods:
# read and write take as an argument either and input or output
#    stream (basically any object with has the method "readline"
#    for reading and "write" for writing)
#    
# readFilename and writeFilename take as an argument a filename
#    in which to read/write the parameters (basically it opens the
#    file and then calles "read" or "write"
#    
# Used by itself is only reads in ONE solution from the file
# for example readFilename will only read the first solution
# Commonly it will be used in a container class only using the
# read and write methods and letting the outside class take care
# of opening the file.

class AUTOSolution(UserDict,Points.Pointset):

    data_keys = set(["PT", "BR", "TY number", "TY", "LAB",
                 "ISW", "NTST", "NCOL", "Active ICP", "rldot",
                 "udotps", "NPARI", "NDIM", "IPS", "IPRIV"])

    long_data_keys = {
        "Parameters": "p",
        "parameters": "p",
        "Parameter NULL vector": "rldot",
        "Free Parameters": "Active ICP",
        "Point number": "PT",
        "Branch number": "BR",
        "Type number": "TY number",
        "Type name": "TY",
        "TY name": "TY",
        "Label": "LAB"}

    def __init__(self,input=None,index=None,name=None,t=None,**kw):
        if isinstance(input,self.__class__):
            for k,v in input.__dict__.items():
                self.__dict__[k] = v
            UserDict.__init__(self,input)
        else:
            UserDict.__init__(self)
            self.c = None
            self.__input          = None
            self.__fullyParsed    = False
            self._dims            = None
            self._mbr             = 0
            self._mlab            = 0
            self.name = name
            self.data.update({"BR":1, "PT":1, "TY number":9, "LAB":0,
                              "ISW":1, "NTST": 1, "NCOL": 0,
                              "NPARI":0, "NDIM": 0, "IPS": None, "IPRIV":0})
            self.indepvarname = 't'
            self.coordnames = []
            self.__parnames = []

            if isinstance(input,(file,gzip.GzipFile)):
                input = fileS(input)
            if isinstance(input,fileS):
                self.read(input,index)
            elif input is not None:
                self.__readarray(input,t)
        self.update(**kw)

    def update(self, dct=None, **kw):
        par = None
        if "constants" in kw:
            self.c = kw["constants"]
            del kw["constants"]
            unames = dict(self.c.get("unames") or [])
            self.coordnames = [unames.get(i+1,'U(%d)'%(i+1))
                               for i in range(len(self.coordnames))]
            parlen = len(self.__parnames)
            parnames = dict(self.c.get("parnames") or [])
            if parnames != {}:
                parlen = max(parlen, max(parnames))
            par = dict(self.c.get("PAR") or [])
            for key in par:
                if key not in parnames.values():
                    parlen = max(key, parlen)
            self.__parnames = [parnames.get(i,"PAR(%d)"%i)
                               for i in range(1,parlen+1)]
            if ((self.name is None or os.path.basename(self.name) == 'fort.8')
                and self.c.get("e") is not None):
                self.name = self.c["e"]

        if self.name is None:
            self.name = ''
        if self.__fullyParsed:
            self.makeIxMaps()
            self.PAR = AUTOParameters(coordnames=self.__parnames,
                                      coordarray=Points.array(self.PAR),
                                      name=self.name)
        if dct is not None:
            for k,v in dct.items():
                self.data[k] = v
        for k,v in kw.items():
            self[k] = v

        if par is not None and not self.__nodata():
            self["PAR"] = par
            u = self.c.get("U")
            if u is not None:
                self["U"] = u

    def __nodata(self):
        return self.__input is None and not self.__fullyParsed

    def __getstate__(self):
        # For pickle: read everything
        self.__readAll()
        return Points.Pointset.__getstate__(self)

    def __str__(self):
        self.__readAll()
        keys = list(self.data)
        for key in ["BR","PT","LAB","TY number","ISW","NTST","NCOL","NDIM",
                    "IPS","IPRIV","NPARI"]:
            keys.remove(key)
        keys.sort()
        rep="  BR    PT  TY  LAB ISW NTST NCOL"
        #add corresponding L2-NORM, etc, from fort.7
        if self["IPS"] is not None:
            rep = rep+" NDIM IPS IPRIV"
        for key in keys:
            if key not in self.data_keys:
                rep = rep+"%19s"%key
        rep=rep+ "\n%4d%6d%4s%5d%4d%5d%5d" % (self["BR"], self["PT"],
                                              self["TY"], self["LAB"],
                                              self["ISW"], self["NTST"],
                                              self["NCOL"])
        if self["IPS"] is not None:
            rep = rep+"%5d%4d%6d"%(self["NDIM"],self["IPS"],self["IPRIV"])
        for key in list(keys):
            if key not in self.data_keys:
                rep = rep+format19_10E3(self[key])
                keys.remove(key)
        if not self.__nodata():
            rep=rep+"\n"+Points.Pointset.__repr__(self)
        for key in keys:
            v = self[key]
            if isinstance(v,Points.Pointset):
                v = repr(v)
            elif type(v) not in [type(1),type(1.0),Points.float64,type("")]:
                v = list(v)
            if type(v) == type([]) and type(v[0]) not in [type(1),type(1.0),
                                                          Points.float64]:
                v = map(str,v)
            rep=rep+"\n"+str(key)+": "+str(v)
        if not self.__nodata():
            rep=rep+"\n"+str(self.PAR)
        return rep

    def __repr__(self):
        result = id(self)
        if result < 0:
            # avoid negative addresses and Python 2.3 warnings
            result += 256 ** struct.calcsize('P')
        return "<_=%s instance at %#010x>"%(self.__class__.__name__,result)

    def __len__(self):
        return Points.Pointset.__len__(self)

    def __setitem__(self,key,value):
        if (type(key) == type("") and not key in self.coordnames and
            key != self.indepvarname and not key in self.__parnames):
            shortkey = self.long_data_keys.get(key,key)
            if shortkey in self.data_keys:
                if shortkey == "TY":
                    value = parseB.reverse_type_translation(value)
                    shortkey = "TY number"
                elif shortkey == "BR":
                    self._mbr = 0
                elif shortkey == "LAB":
                    self._mlab = 0
                self.data[shortkey] = value
                return
            if shortkey == "PAR":
                if type(value) == type({}):
                    value = value.items()
                for k,v in value:
                    if isinstance(k,str):
                        self.PAR[k] = v
                    else:
                        self.PAR[k-1] = v
                return
            if shortkey == "p":
                self.PAR = AUTOParameters(coordnames=self.__parnames,
                                          coordarray=value, name=self.name)
                return
            if shortkey == "U":
                if type(value) == type({}):
                    value = value.items()
                for i,(k,v) in enumerate(value):
                    if isinstance(k,str):
                        value[i] = self.coordnames.index(k)
                if len(self.coordarray[0]) > 1:
                    # reduce solution to one point
                    del self.coordarray
                    del self.indepvararray
                    self.coordarray = Points.N.array([[0.0]]*max(dict(value)))
                    self.indepvararray = Points.N.array([0.0])
                    self.data.update({"NTST": 1, "NCOL": 0})
                    del self.data["Active ICP"]
                    del self.data["rldot"]
                    del self.data["udotps"]
                for k,v in value:
                    self.coordarray[k-1,0] = v
                return
        try:
            Points.Pointset.__setitem__(self,key,value)
        except (TypeError, ValueError, KeyError, IndexError):
            if self.__nodata():
                raise AUTOExceptions.AUTORuntimeError("Unknown option: %s"%key)
            self.PAR[key] = value

    def __getitem__(self,key):
        big_data_keys = ["data","Active ICP","rldot","p","udotps"]
        if (isinstance(key,str) and key not in self.coordnames and
            key != self.indepvarname and key not in self.__parnames):
            shortkey = self.long_data_keys.get(key,key)
            if shortkey in big_data_keys:
                self.__readAll()
            if shortkey in self.data_keys:
                if shortkey == "TY":
                    return parseB.type_translation(
                        self.data["TY number"])["short name"]
                return self.data[shortkey]
            if shortkey == "p":
                return self.PAR
            if shortkey == "data":
                return self
        if isinstance(key,str) and hasattr(self,"b") and key in self.b:
            if not self.__fullyParsed or not key in self.PAR:
                if key not in self.coordnames:
                    return self.b[key]
        self.__readAll()
        if isinstance(key, str):
            try:
                return Points.Pointset.__getitem__(self,key)
            except:
                try:
                    return self.PAR[key]
                except:
                    return self.data[key]
        ret = Points.Pointset.__getitem__(self,key)
        if not isinstance(key, int):
            return ret
        return SLPoint(ret, self, key)

    def __call__(self, p=None, coords=None):
        if p is None:
            return(str(self))
        return Points.Pointset.__call__(self, p, coords)

    def __copy__(self):
        return self.__class__(self)

    def copy(self):
        return self.__copy__()

    def __contains__(self,key):
        return (key in ["data","TY"] or key in self.long_data_keys or
                key in self.__parnames or
                (not self.__fullyParsed and key in self.data_keys) or
                (self.__fullyParsed and key in self.data) or
                (hasattr(self,"b") and key in self.b) or
                Points.Pointset.has_key(self,key))

    def has_key(self, key):
        return self.__contains__(key)

    def get(self, key, failobj=None):
        if key in self:
            return self[key]
        return failobj

    def type(self):
        return parseB.type_translation(self["Type number"])["long name"]

    def load(self,**kw):
        """Load solution with the given AUTO constants.
        Returns a shallow copy with a copied set of updated constants
        """
        constants = kw.get("constants")
        if "constants" in kw:
            del kw["constants"]
        c = parseC.parseC(self.c)
        datakw = {}
        for key in self.data_keys:
            if key in kw and (key not in c or key in ["LAB", "TY"]):
                datakw[key] = kw[key]
                if key not in ["LAB", "TY"]:
                    del kw[key]
        oldirs = c["IRS"]
        c.update(constants, **kw)
        if oldirs is not None and c["IRS"] == 0:
            return AUTOSolution(constants=c, **datakw)
        if self["LAB"] != 0:
            c["IRS"] = self["LAB"]
        return AUTOSolution(self, constants=c, **datakw)

    def run(self,**kw):
        """Run AUTO.

        Run AUTO from the solution with the given AUTO constants.
        Returns a bifurcation diagram of the result.
        """
        from auto import runAUTO
        return runAUTO.runAUTO(selected_solution=self.load(**kw)).run()

    def readAllFilename(self,filename):
        inputfile = fileS(filename)
        self.readAll(inputfile)
        inputfile.close()

    def readFilename(self,filename):
        inputfile = fileS(filename)
        self.read(inputfile)
        inputfile.conditionalclose()
        # else don't close the input file but garbage collect

    def writeFilename(self,filename,mlab=False):
        output = open(filename,"wb")
        self.write(output,mlab)
        output.flush()
        output.close()

    def writeRawFilename(self,filename):
        output = open(filename,"w")
        self.writeRaw(output)
        output.flush()
        output.close()
        
    def toArray(self):
        return [
            [vector["t"]] + [point for point in vector["u"]]
            for vector in self["data"]]

    def writeRaw(self,output):
        s = "%24.15E"*(self.coordarray.shape[0]+1)
        for i in range(self.coordarray.shape[1]):
            output.write(s%((self.indepvararray[i],)+
                            tuple(self.coordarray[:,i]))+"\n")
            
    def read(self, inputfile=None, index=0):
        if self.__fullyParsed:
            return
        if inputfile is None:
            # read data into memory
            self.__input.readstr(self.__index)
            self.__input.conditionalclose()
            return
        if not isinstance(inputfile, fileS):
            inputfile = fileS(inputfile)
        self.__input = inputfile
        self.__index = index
        self.__readHeader()
    
    def readAll(self, inputfile):
        self.read(inputfile)
        self.__readAll()

    def __readHeader(self):
        header = self.__input.solutions[self.__index]['header']
        self.indepvarname = 't'
        self.__numEntriesPerBlock = header[7]
        ndim = self.__numEntriesPerBlock-1
        if ndim < len(self.coordnames):
            self.coordnames = self.coordnames[:ndim]
        for i in range(len(self.coordnames),
                       self.__numEntriesPerBlock-1):
            self.coordnames.append("U(%d)"%(i+1))

        self.update({"NPARI": 0, "NDIM": 0, "IPS": None, "IPRIV": 0})
        for i, key in enumerate(["BR", "PT", "TY number", "LAB", "",
                                 "ISW", "", "", "", "NTST",
                                 "NCOL", "", "NPARI", "NDIM", "IPS",
                                 "IPRIV"]):
            if key and i < len(header):
                self[key] = header[i]
        self.__numChangingParameters = header[4]
        self.__numSValues = header[6]
        self.__numLinesPerEntry = header[8]
        self.__numFreeParameters = header[11]
        
    def __readAll(self):
        if self.__fullyParsed or self.__nodata():
            return
        if not Points.numpyimported:
            Points.importnumpy()
        fromstring = Points.fromstring
        N = Points.N
        self.__fullyParsed = True
        n = self.__numEntriesPerBlock
        nrows = self.__numSValues
        total = n * nrows + self.__numFreeParameters
        nlinessmall = (((n-1)/7+1) * nrows + (self.__numFreeParameters+6)/7)
        if self["NTST"] != 0 and self.__numLinesPerEntry > nlinessmall:
            total += 2 * self.__numChangingParameters + (n-1) * nrows
        fdata = self.__input.readfloats(self.__index, total)
        ups = N.reshape(fdata[:n * nrows],(nrows,n))
        self.indepvararray = ups[:,0]
        self.coordarray = N.transpose(ups[:,1:])
        j = n * nrows

        # Check if direction info is given
        if self["NTST"] != 0 and self.__numLinesPerEntry > nlinessmall:
            nfpr = self.__numChangingParameters
            self["Active ICP"] = list(map(int,fdata[j:j+nfpr]))
            j = j + nfpr
            self["rldot"] = fdata[j:j+nfpr]
            j = j + nfpr
            n = n - 1
            self["udotps"] = N.transpose(
                N.reshape(fdata[j:j+n * self.__numSValues],(-1,n)))
            udotnames = ["UDOT(%d)"%(i+1) for i in
                         range(self.__numEntriesPerBlock-1)]
            self["udotps"] = Points.Pointset({
                "coordarray": self["udotps"],
                "coordnames": udotnames,
                "name": self.name})
            self["udotps"]._dims = None
            j = j + n * nrows

        self.PAR = fdata[j:j+self.__numFreeParameters]
        Points.Pointset.__init__(self,{
                "indepvararray": self.indepvararray,
                "indepvarname": self.indepvarname,
                "coordarray": self.coordarray,
                "coordnames": self.coordnames,
                "name": self.name})
        self.update()

    def __readarray(self,coordarray,indepvararray=None):
        #init from array
        if not Points.numpyimported:
            Points.importnumpy()        
        N = Points.N
        if not hasattr(coordarray[0],'__len__'):
            # point
            indepvararray = [0.0]
            ncol = 0
            ntst = 1
            coordarray = [[d] for d in coordarray]
            pararray = []
        else:
            # time + solution
            if indepvararray is None:
                indepvararray = coordarray[0]
                coordarray = coordarray[1:]
            ncol = 1
            ntst = len(indepvararray)-1
            t0 = indepvararray[0]
            period = indepvararray[-1] - t0
            if period != 1.0 or t0 != 0.0:
                #scale to [0,1]
                for i in range(len(indepvararray)):
                    indepvararray[i] = (indepvararray[i] - t0)/period
            # set PAR(11) to period
            pararray = 10*[0.0] + [period]
        indepvarname = "t"
        ndim = len(coordarray)
        coordnames = ["U(%d)"%(i+1) for i in range(ndim)]
        Points.Pointset.__init__(self,{"indepvararray": indepvararray,
                                       "indepvarname": indepvarname,
                                       "coordarray": coordarray,
                                       "coordnames": coordnames})
        self.__fullyParsed = True
        self.data.update({"NTST": ntst, "NCOL": ncol, "LAB": 1, "NDIM": ndim})
        self.__numChangingParameters = 1
        self.PAR = pararray

    def __getattr__(self,attr):
        if self.__nodata():
            raise AUTOExceptions.AUTORuntimeError("Solution without data.")
        if not self.__fullyParsed and attr != "__del__":
            self.__readAll()
            return getattr(self,attr)
        raise AttributeError(attr)

    def write(self,output,mlab=False):
        if self.__nodata():
            return
        try:
            "".encode("ascii") + ""
            #write encoded
            def write_enc(s):
                output.write(s)
        except TypeError: #Python 3.0
            if hasattr(output, "encoding"):
                # output is a text stream
                if os.linesep == "\n":
                    def write_enc(s):
                        output.write(s)
                else:
                    def write_enc(s):
                        output.write(s.replace(os.linesep, "\n"))
            else:
                # output is a binary stream (common case)
                def write_enc(s):
                    output.write(s.encode("ascii"))

        if self.__fullyParsed:
            ndim = len(self.coordarray)
            npar = len(self["Parameters"])
            ntpl = len(self)
            nrowpr = (ndim//7+1) * ntpl + (npar+6)//7
            nfpr = self.__numChangingParameters
            if "Active ICP" in self.data:
                nfpr = len(self.get("Active ICP",[0]))
                nrowpr += (nfpr+19)//20 + (nfpr+6)//7 + (ndim+6)//7 * ntpl
        else:
            ndim = self.__numEntriesPerBlock-1
            npar = self.__numFreeParameters
            ntpl = self.__numSValues
            nfpr = self.__numChangingParameters
            nrowpr = self.__numLinesPerEntry

        line = "%6d%6d%6d%6d%6d%6d%8d%6d%8d%5d%5d%5d" % (self["BR"],
                                                         self["PT"],
                                                         self["TY number"],
                                                         self["LAB"],
                                                         nfpr,
                                                         self["ISW"],
                                                         ntpl,
                                                         ndim+1,
                                                         nrowpr,
                                                         self["NTST"],
                                                         self["NCOL"],
                                                         npar
                                                         )
        if self["IPS"] is not None:
            line += "%5d%5d%5d%5d" % (self["NPARI"],self["NDIM"],self["IPS"],
                                      self["IPRIV"])
        write_enc(line+os.linesep)
        # If the file isn't already parsed, we can just copy from the input
        # file into the output file
        if not self.__fullyParsed:
            inputsolution = self.__input.readstr(self.__index)
            if hasattr(output, "encoding") and hasattr(inputsolution, "decode"):
                inputsolution = inputsolution.decode("ascii")
            output.write(inputsolution)
        # Otherwise we do a normal write.  NOTE: if the solution isn't already
        # parsed it will get parsed here.
        else:
            slist = []
            for i in range(len(self.indepvararray)):
                slist.append("    "+format19_10E3(self.indepvararray[i]))
                for j in range(1,len(self.coordarray)+1):
                    if j%7==0:
                        slist.append(os.linesep+"    ")
                    slist.append(format19_10E3(self.coordarray[j-1,i]))
                slist.append(os.linesep)
            write_enc("".join(slist))
            if "Active ICP" in self.data:
                # Solution contains derivative information.
                j = 0
                for parameter in self["Active ICP"]:
                    write_enc("%5d" % (parameter))
                    j = j + 1
                    if j%20==0:
                        write_enc(os.linesep)
                if j%20!=0:
                    write_enc(os.linesep)

                line = "    "
                i = 0
                for vi in self["rldot"]:
                    num = format19_10E3(vi)
                    if i != 0 and i%7==0:
                        line = line + os.linesep + "    "
                    line = line + num
                    i = i + 1
                write_enc(line+os.linesep)

                # write UDOTPS
                slist = []
                c = self["udotps"].coordarray
                l = len(c)
                for i in range(len(self.indepvararray)):
                    slist.append("    ")
                    for j in range(len(self.coordarray)):
                        if j!=0 and j%7==0:
                            slist.append(os.linesep+"    ")
                        if j<l:
                            slist.append(format19_10E3(c[j,i]))
                        else:
                            slist.append(format19_10E3(0))
                    slist.append(os.linesep)
                write_enc("".join(slist))

            line = "    "
            j = 0
            for parameter in self.PAR.toarray():
                num = format19_10E3(parameter)
                line = line + num 
                j = j + 1
                if j%7==0:
                    write_enc(line+os.linesep)
                    line = "    "
            if j%7!=0:
                write_enc(line+os.linesep)
        if mlab and (self._mbr > 0 or self._mlab > 0) and not (
            self._mbr == self["BR"] and self._mlab == self["LAB"]):
            # header for empty solution so that AUTO can pickup the maximal
            # label and branch numbers.
            if self["IPS"] is not None:
                write_enc("%6d%6d%6d%6d%6d%6d%8d%6d%8d%5d%5d%5d%5d%5d%5d%5d%s"%
                      ((self._mbr, 0, 0, self._mlab) + 12*(0,) + (os.linesep,)))
            else:
                write_enc("%6d%6d%6d%6d%6d%6d%8d%6d%8d%5d%5d%5d%s"%
                      ((self._mbr, 0, 0, self._mlab) + 8*(0,) + (os.linesep,)))
        output.flush()

def pointtest(a,b):
    keys = ['Type number', 'Type name', 'Parameter NULL vector',
            'Free Parameters', 'Branch number',
            'data', 'NCOL', 'Label', 'ISW', 'NTST',
            'Point number', 'Parameters']

    # make sure the solutions are fully parsed...
    scratch=a['Parameters']
    scratch=b['Parameters']
    for key in keys:
        if key not in a:
            raise AUTOExceptions.AUTORegressionError("No %s label"%(key,))
    if len(a["data"]) != len(b["data"]):
        raise AUTOExceptions.AUTORegressionError("Data sections have different lengths")


def test():
    print("Testing reading from a filename")
    foo = parseS()
    foo.readFilename("test_data/fort.8")    
    if len(foo) != 5:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest(foo.getIndex(0),foo.getIndex(3))

    print("Testing reading from a stream")
    foo = parseS()
    fp = open("test_data/fort.8","rb")
    foo.read(fp)    
    if len(foo) != 5:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest(foo.getIndex(0),foo.getIndex(3))

    
    
    print("parseS passed all tests")

if __name__ == '__main__' :
    test()



from AUTOclui import *

def get(solution, eps=1e-3):
    #construct a new constant solution based on the flq solution
    PAR = solution.PAR
    PAR['eps'] = eps
    u = []
    s0 = solution(0)
    for i in range(6):
        u.append(s0[i]+eps*s0[i+6])
    p = {2:PAR(2),3:PAR(3),6:PAR(6),12:0}
    # init PAR(21:23) to u[:3]
    for i in range(3):
        p[21+i] = u[i]
    # init PAR(25:30) to USTART and PAR(31:36) to VSTART
    for i in range(12):
        p[25+i] = s0[i]
    return load(u,PAR=p)

def create(sfile='flq',label=2):
    solution = loadbd(s=sfile)(label)
    save(get(solution,eps),'startman')

# This is the Python syntax for making a script runable    
if __name__ == '__main__':
    if len(sys.argv) > 2:
        create(sys.argv[1], int(sys.argv[2]))
    elif len(sys.argv) > 1:
        create(sys.argv[1])
    else:
        create()




#! /usr/bin/env python
from auto import AUTOutil
import sys
import os
from auto import AUTOCommands
from auto import runAUTO
from auto import interactiveBindings
try:
    import __builtin__
    from new import function
except ImportError:
    import builtins as __builtin__ # Python 3
    from types import FunctionType

class AUTOSimpleFunctions:
    def __init__(self,outputRecorder=None,errorRecorder=None):
        
        # Initialize the global AUTO runner
        runner = runAUTO.runAUTO()
        if outputRecorder is not None:
            stdout = sys.stdout
            class WriteLog(object):
                def write(self,s):
                    outputRecorder.write(s)
                    stdout.write(s)
                def flush(self):
                    outputRecorder.flush()
                    stdout.flush()
            runner.config(log=WriteLog())
        if errorRecorder is not None:
            stderr = sys.stderr
            class WriteErr(object):
                def write(self,s):
                    errorRecorder.write(s)
                    stderr.write(s)
                def flush(self):
                    errorRecorder.flush()
                    stderr.flush()
            runner.config(err=WriteErr())
        self._runner = runner

        # Read in the aliases.
        self._aliases = None

        parser = AUTOutil.getAUTORC()
        if parser.has_section("AUTO_command_aliases"):
            self._aliases = {}
            for option in parser.options("AUTO_command_aliases"):
                cmd = parser.get("AUTO_command_aliases",option)
                if cmd not in self._aliases:
                    self._aliases[cmd] = []
                self._aliases[cmd].append(option)

        self._addCommands([AUTOCommands])

        # Now I resolve the aliases
        for key, aliases in self._aliases.items():
            for alias in aliases:
                f = self._copyfunction(getattr(AUTOCommands,key).fun, alias)
                setattr(self, alias, f)
                doc = getattr(AUTOCommands,key).__doc__
                doc = self._adjustdoc(doc, alias, key)
                f.__doc__ = doc

    def _copyfunction(self, f, key):

        def withrunner(runner=None):
            return self._runner or runner

        if 'FunctionType' in globals():
            func_globals = f.__globals__.copy()
            func_globals["withrunner"] = withrunner
            return FunctionType(f.__code__, func_globals, key,
                                f.__defaults__, f.__closure__)
        else:
            func_globals = f.func_globals.copy()
            func_globals["withrunner"] = withrunner
            return function(f.func_code, func_globals, key,
                            f.func_defaults or ())

    def _adjustdoc(self, doc, commandname, truecommandname = None):
        # If we were created with the nonempty string return a formatted
        # reference for the given command as the string
        if doc == None:
            return doc
        # Get rid of the LaTeX stuff from the string that gets returned.
        doc = doc.replace("\\begin{verbatim}","")
        doc = doc.replace("\\end{verbatim}","")
        doc = doc + "\n"

        doc = doc.replace("FUNC", commandname)
        # This means help was asked for an alias
        if not truecommandname is None:
            commandname = truecommandname
            doc = doc + "Command name: "+commandname+"\n"
        doc = doc + "Aliases: "
        if commandname in self._aliases:
            doc = doc + " ".join(self._aliases[commandname])
        return doc

    def _addCommands(self,moduleList):
        addaliases = self._aliases is None
        if addaliases:
            self._aliases = {}
        for module in [AUTOCommands]:
            # Now we copy the commands from the module
            for key in module.__dict__:
                cmd = getattr(module,key)
                # Check to see if it is a command
                if hasattr(cmd,"fun"):
                    if addaliases and cmd.alias is not None:
                        self._aliases[key] = [cmd.fun.__name__] + cmd.alias
                    f = self._copyfunction(cmd.fun, key)
                    setattr(self, key, f)
                    doc = cmd.__doc__
                    doc = self._adjustdoc(doc, key)
                    f.__doc__ = doc

# Export the functions inside AUTOSimpleFunctions in a dictionary
# This also allows the setting of the log
def exportFunctions(log=None,err=None):
    AUTOSimpleFunctionsInstance = AUTOSimpleFunctions(log,err)
    dict = {}
    for name in AUTOSimpleFunctionsInstance.__dict__:
        dict[name] = getattr(AUTOSimpleFunctionsInstance, name)
    return dict

# Export the functions inside AUTOSimpleFunctions in this modules namespace
# This is to make "from AUTOclui import *" work
# ALMOST like the AUTOInteractiveConsole.  Things that
# don't work are help, shell, !, ls, cd, and any changes
# to the aliases
if "AUTO_DIR" not in os.environ:
    absfile = os.path.abspath(__file__)
    autodir = os.path.dirname(os.path.dirname(os.path.dirname(absfile)))
    os.environ["AUTO_DIR"] = autodir
funcs = exportFunctions()
runner = interactiveBindings.AUTOInteractiveConsole(funcs)
for name,value in funcs.items():
    if name not in __builtin__.__dict__:
        globals()[name] = value

def test():
    interactiveBindings._testFilename("../demos/python/fullTest.auto","test_data/fullTest.log")

if __name__ == "__main__":
    test()










#! /usr/bin/env python
#    Visualization for Bifurcation Manifolds
#    Copyright (C) 1997 Randy Paffenroth and John Maddocks
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU  General Public
#    License as published by the Free Software Foundation; either
#    version 2 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this library; if not, write to the Free
#    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
#    MA 02111-1307, USA
from auto import parseB
from auto import parseS
from auto import parseD
from auto import AUTOExceptions

class parseBandS:
    def __init__(self,fort7_filename=None,fort8_filename=None,fort9_filename=None):
        self.diagnostics = None
        if isinstance(fort7_filename, list):
            self.diagram = None
            self.solution = parseS.parseS(fort7_filename)
        elif fort8_filename is None and fort7_filename is not None:
            raise AUTOExceptions.AUTORuntimeError("Must set both both filenames")
        elif isinstance(fort7_filename, str) or fort7_filename is None:
            self.diagram = parseB.parseB(fort7_filename)
            self.solution = parseS.parseS(fort8_filename)
            if not fort9_filename is None:
                self.diagnostics = parseD.parseD(fort9_filename)
        else:
            self.diagram = fort7_filename
            self.solution = fort8_filename
            self.diagnostics = fort9_filename
        if self.diagnostics is None:
            self.diagnostics = []
        self.dg = self.diagram
        self.sl = self.solution

    def __str__(self):
        return self.diagram.summary()
        
    def __repr__(self):
        return ""
        
    def getLabel(self,label):
        if isinstance(label, int):
            return self.solution(label)
        return parseBandS(self.diagram(label),self.solution(label),
                          self.diagnostics)

    def __add__(self,x):
        if x == []:
            x = parseBandS()
        add = parseBandS(self.diagram + x.diagram,
                         self.solution + x.solution,
                         self.diagnostics + x.diagnostics)
        add.uniquelyLabel()
        return add

    def __radd__(self,x):
        if x == []:
            x = parseBandS()
        add = parseBandS(x.diagram + self.diagram,
                         x.solution + self.solution,
                         x.diagnostics + self.diagnostics)
        add.uniquelyLabel()
        return add

    def __iadd__(self,x):
        if x != []:
            self.diagram.__iadd__(x.diagram)
            self.solution.__iadd__(x.solution)
            self.diagnostics.__iadd__(x.diagnostics)
        self.uniquelyLabel()
        return self

    def extend(self,x):
        self.__iadd__(x)

    def read(self,fort7_input,fort8_input,fort9_input=None):
        self.diagram.read(fort7_input)
        self.solution.read(fort8_input)
        if not fort9_input is None:
            self.diagnostics.read(fort9_input)

    def write(self,fort7_output,fort8_output,fort9_output=None):
        self.diagram.write(fort7_output)
        self.solution.write(fort8_output)
        if not fort9_output is None:
            self.diagnostics.write(fort9_output)

    def save(self,output_filename):
        self.writeFilename('b.'+output_filename,
                           's.'+output_filename,
                           'd.'+output_filename)
        return self

    def readFilename(self,fort7_filename,fort8_filename,fort9_filename=None):
        self.diagram.readFilename(fort7_filename)
        self.solution.readFilename(fort8_filename)
        if not fort9_filename is None:
            self.diagnostics.readFilename(fort9_filename)

    def writeFilename(self,fort7_filename,fort8_filename=None,fort9_filename=None):
        #if only one filename is given, then just save the solutions file
        if fort8_filename is None:
            fort8_filename = fort7_filename
        else:
            self.diagram.writeFilename(fort7_filename)
        self.solution.writeFilename(fort8_filename)
        if not fort9_filename is None:
            self.diagnostics.writeFilename(fort9_filename)

    def deleteLabel(self,label=None,keepTY=0,keep=0):
        self.diagram.deleteLabel(label,keepTY,keep)
        self.solution.deleteLabel(label,keep)

    def relabel(self,old_label=None,new_label=None):
        if old_label is None and new_label is None:
            self.diagram.uniquelyLabel()
            self.solution.uniquelyLabel()
            return self
        self.diagram.relabel(old_label,new_label)
        self.solution.relabel(old_label,new_label)
    
    def uniquelyLabel(self):
        self.diagram.uniquelyLabel()
        self.solution.uniquelyLabel()

    def getLabels(self):
        return self.diagram.getLabels()

def pointtest7(a,b):
    if "TY name" not in a:
        raise AUTOExceptions.AUTORegressionError("No TY name label")
    if "TY number" not in a:
        raise AUTOExceptions.AUTORegressionError("No TY number label")
    if "BR" not in a:
        raise AUTOExceptions.AUTORegressionError("No BR label")
    if "data" not in a:
        raise AUTOExceptions.AUTORegressionError("No data label")
    if "PT" not in a:
        raise AUTOExceptions.AUTORegressionError("No PT label")
    if "LAB" not in a:
        raise AUTOExceptions.AUTORegressionError("No LAB label")
    if len(a["data"]) != len(b["data"]):
        raise AUTOExceptions.AUTORegressionError("Data sections have different lengths")
   
    
def pointtest8(a,b):
    keys = ['Type number', 'Type name',
            'Free Parameters',  'Branch number', 'Parameter NULL vector',
            'data', 'NCOL', 'Label', 'ISW', 'NTST',
            'Point number', 'Parameters']

    scratch=a['Parameters']
    scratch=b['Parameters']
    for key in keys:
        if key not in a:
            raise AUTOExceptions.AUTORegressionError("No %s label"%(key,))
    if len(a["data"]) != len(b["data"]):
        raise AUTOExceptions.AUTORegressionError("Data sections have different lengths")


def test():
    foo = parseBandS()
    foo.readFilename("test_data/fort.7","test_data/fort.8")
    if len(foo.diagram) != 150:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest7(foo.diagram.getIndex(0),foo.diagram.getIndex(57))
    if len(foo.solution) != 5:
        raise AUTOExceptions.AUTORegressionError("File length incorrect")
    pointtest8(foo.solution.getIndex(0),foo.solution.getIndex(3))

    if len(foo.getLabels()) != 5:
        raise AUTOExceptions.AUTORegressionError("Incorrect number of labels")

    print("Deleting labels")
    foo.deleteLabel(range(6,9))
    
    if len(foo.getLabels()) != 2:
        raise AUTOExceptions.AUTORegressionError("Incorrect number of labels")

    print("Relabeling")
    foo.relabel(9,57)

    for i in range(len(foo.diagram)):
        if foo.diagram.getIndex(0)["TY number"] != 0:
            if foo.diagram.getIndex(0)["LAB"] != 57:
                raise AUTOExceptions.AUTORegressionError("Incorrect label")
            break
    if foo.solution.getIndex(0)["Label"] != 57:
        raise AUTOExceptions.AUTORegressionError("Incorrect label")

    print("Making labels unique")
    foo.uniquelyLabel()

    for i in range(len(foo.diagram)):
        if foo.diagram.getIndex(0)["TY number"] != 0:
            if foo.diagram.getIndex(0)["LAB"] != 1:
                raise AUTOExceptions.AUTORegressionError("Incorrect label")
            break
    if foo.solution.getIndex(0)["Label"] != 1:
        raise AUTOExceptions.AUTORegressionError("Incorrect label")

    print("parseBandS passed all tests")

if __name__ == '__main__' :
    test()



#! /usr/bin/env python
try:
    from auto.graphics import grapher_mpl as grapher
except ImportError:
    from auto.graphics import grapher
    print("Using plain TkInter for plotting. You can obtain better quality graphics")
    print("using matplotlib (http://matplotlib.sf.net).")
from auto import parseB
from auto import parseS
from auto import parseC
from auto import AUTOutil
import os
import gc
import sys
Axes3D = grapher.Axes3D

class plotter(grapher.GUIGrapher):
    def __init__(self,parent=None,**kw):

        optionDefaults = {}
        # The kind of diagram (single solution vs. bifur diagram)
        optionDefaults["type"]     = ("bifurcation",self.__optionCallback)  
        # The X column
        optionDefaults["bifurcation_x"] = ([0],self.__optionCallback)
        optionDefaults["solution_x"]    = ([-1],self.__optionCallback)
        # The Y column
        optionDefaults["bifurcation_y"] = ([1],self.__optionCallback)
        optionDefaults["solution_y"]    = ([0],self.__optionCallback)
        # The Z column
        optionDefaults["bifurcation_z"] = (None,self.__optionCallback)
        optionDefaults["solution_z"]    = (None,self.__optionCallback)
        # The coordinate names
        optionDefaults["bifurcation_coordnames"] = (None,self.__optionCallback)
        optionDefaults["solution_indepvarname"]  = (None,self.__optionCallback)
        optionDefaults["solution_coordnames"]    = (None,self.__optionCallback)
        optionDefaults["labelnames"]             = (None,self.__optionCallback)
        # Sets of labels that the user is likely to want to use
        optionDefaults["label_defaults"]   = (None,self.__optionCallback)
        # Sets of columns that the user is likely to want to use
        optionDefaults["bifurcation_column_defaults"]   = (None,self.__optionCallback)
        optionDefaults["solution_column_defaults"]   = (None,self.__optionCallback)
        # The index of the solution we wish to draw
        optionDefaults["index"]    = ([0],self.__optionCallback)
        # The label of the solution we wish to draw
        optionDefaults["label"]    = ([0],self.__optionCallback)

        # Already parsed data structures
        optionDefaults["bifurcation_diagram"]          = (parseB.parseB(),self.__optionCallback)
        optionDefaults["solution"]          = (parseS.parseS(),self.__optionCallback)
        optionDefaults["bifurcation_diagram_filename"] = ("",self.__optionCallback)
        optionDefaults["solution_filename"] = ("",self.__optionCallback)
        optionDefaults["runner"]         = (None,self.__optionCallback)
        optionDefaults["mark_t"]         = (None,self.__optionCallback)

        optionDefaults["letter_symbols"] = (True,self.__optionCallback)
        optionDefaults["bifurcation_symbol"]     = ("B",self.__optionCallback)
        optionDefaults["limit_point_symbol"]     = ("L",self.__optionCallback)
        optionDefaults["hopf_symbol"]            = ("H",self.__optionCallback)
        optionDefaults["zero_hopf_symbol"]       = ("ZH",self.__optionCallback)
        optionDefaults["bogdanov_takens_symbol"] = ("BT",self.__optionCallback)
        optionDefaults["cusp_symbol"]            = ("CP",self.__optionCallback)
        optionDefaults["generalized_hopf_symbol"]= ("GH",self.__optionCallback)
        optionDefaults["1_1_resonance_symbol"]   = ("R1",self.__optionCallback)
        optionDefaults["1_2_resonance_symbol"]   = ("R2",self.__optionCallback)
        optionDefaults["1_3_resonance_symbol"]   = ("R3",self.__optionCallback)
        optionDefaults["1_4_resonance_symbol"]   = ("R4",self.__optionCallback)
        optionDefaults["fold_flip_symbol"]       = ("LPD",self.__optionCallback)
        optionDefaults["fold_torus_symbol"]      = ("LTR",self.__optionCallback)
        optionDefaults["flip_torus_symbol"]      = ("PTR",self.__optionCallback)
        optionDefaults["torus_torus_symbol"]     = ("TTR",self.__optionCallback)
        optionDefaults["period_doubling_symbol"] = ("D",self.__optionCallback)
        optionDefaults["torus_symbol"]           = ("T",self.__optionCallback)
        optionDefaults["user_point_symbol"]      = ("U",self.__optionCallback)
        optionDefaults["error_symbol"]           = ("X",self.__optionCallback)

        optionDefaults["ps_colormode"]           = ("color",self.__optionCallback)
        optionDefaults["stability"]              = (False,self.__optionCallback)
        optionDefaults["coloring_method"]        = ("curve",self.__optionCallback)

        parser = AUTOutil.getAUTORC("AUTO_plotter")
        optionDefaultsRC = {}
        c = parseC.parseC()
        for option in parser.options("AUTO_plotter"):
            try:
                optionDefaultsRC[option] = self.parseoption(
                    option,parser.get("AUTO_plotter",option),c)
            except:
                optionDefaultsRC[option] = self.parseoption(
                    option,parser.get("AUTO_plotter",option,raw=True),c)
        # Let these override the RC options, if specified.
        for key in ["hide","xlabel","ylabel","zlabel"]:
            if key in kw:
                optionDefaultsRC[key] = kw[key]

        self.__needsPlot = None
        grapher.GUIGrapher.__init__(self,parent,**optionDefaultsRC)

        self.addOptions(**optionDefaults)
        self.addRCOptions(**optionDefaultsRC)
        for options in [optionDefaultsRC, kw]:
            if "letter_symbols" in options:
                self.__optionCallback("letter_symbols",
                                      options["letter_symbols"], options)
                del options["letter_symbols"]
        plotter._configNoDraw(self,**optionDefaultsRC)
        plotter._configNoDraw(self,**kw)
        self._plotNoDraw()
        self.__needsPlot = None
        for coord in 'x', 'y', 'z':
            if "min"+coord not in kw or "max"+coord not in kw:
                self.computeRange(coord,kw.get("min"+coord),
                                  kw.get("max"+coord))
        grapher.GUIGrapher.plot(self)

    def parselist(self,option,v,c,sep):
        quote = ' '
        i = 0
        vlist = []
        level = 0
        for j, ch in enumerate(v):
            if ch == '[' and level == 0 and option[-9:] == "_defaults":
                level += 1
                vlist1 = []
                i = j+1
                continue
            if level == 1 and quote == ' ' and ch == ']':
                s = v[i:j]
                i = j+1
            elif j == len(v) - 1:
                s = v[i:]
            elif ch in sep and quote == ' ':
                s = v[i:j]
                i = j+1
            else:
                if quote == ' ':
                    if ch in ['"',"'"]:
                        quote = ch
                elif ch == quote:
                    quote = ' '
                continue
            s = s.strip()
            if len(s) == 0 or s[0] not in ['"',"'"]:
                try:
                    int(s)
                except ValueError:
                    s = "'"+s+"'"
            if level == 1:
                vlist1.append(s)
                if ch == ']':
                    level -= 1
                    vlist1 = c.scanvalue("["+",".join(vlist1)+"]")[0]
                    for i1, l1 in enumerate(vlist1):
                        try:
                            vlist1[i1] = int(l1)
                        except ValueError:
                            pass
                    vlist.append(vlist1)
            elif option[-9:] != "_defaults":
                vlist.append(s)
        if option[-9:] == "_defaults":
            return vlist
        v = c.scanvalue("["+",".join(vlist)+"]")[0]
        for i, l in enumerate(v):
            try:
                v[i] = int(l)
            except ValueError:
                pass
        return v

    def parseoption(self,option,v,c):
        special = {'None': None, 'True': True, 'False': False,
                   'Yes': True, 'No': False}
        i = 0
        quoted = False
        while i < len(v) and v[i] in ['"',"'"]:
            q = v[i]
            # look for endquote
            j = v.find(q,i+1)
            if j < 0:
                break
            # remove quotes
            quoted = True
            v = v[:i]+v[i+1:j]+v[j+1:]
            i = j - 1
        # remove comment after last quote
        i = v.find('#',i+1)
        if i >= 0:
            v = v[:i].strip()
        if quoted:
            return v
        w = v.capitalize()
        if w in special:
            v = special[w]
        elif (option[-2:] in ['_x','_y','_z'] or
              option[-9:] == "_defaults" or
              option[-11:] == "_coordnames" or
              option in ["label","index","dashes","labelnames"] or
              (option[0] == 'd' and len(option) == 2)):
            # convert to list, quote args if not numbers,
            # then use the same method as for constant files
            if ((option[0] == 'd' and len(option) == 2) or
                option == "labelnames"):
                brackets = ['{']
                sep = [',',':']
            else:
                brackets = ['[','(']
                sep = [',']
            if len(v) > 0 and v[0] in brackets:
                endbracket = {'{':'}','[':']','(':')'}[v[0]]
                v = v[1:]
                if len(v) > 0 and v[-1] == endbracket:
                    v = v[:-1]
            v = self.parselist(option,v,c,sep)
            if ((option[0] == 'd' and len(option) == 2) or
                option == "labelnames"):
                v = dict([(v[i],v[i+1]) for i in range(0,len(v),2)])
                if option != "labelnames":
                    # Convert "True"/"False"/"yes"/"no" to True/False
                    for k in v:
                        try:
                            w = v[k].capitalize()
                            if w in special:
                                v[k] = special[w]
                        except AttributeError:
                            pass
        else: # simple value
            try:
                v = int(v)
            except ValueError:
                try:
                    v = float(v)
                except ValueError:
                    pass
        return v

    def config(self,cnf=None,**kw):
        rval = self._configNoDraw(cnf,**kw)
        if isinstance(cnf, str) or (cnf is None and not kw):
            return rval
        if 'auto.graphics.grapher_mpl' not in sys.modules and not hasattr(self, "tk"):
            # plotting disabled
            return
        if self.__needsPlot:
            self._plotNoDraw()
            self.__needsPlot = None
            self.clear()
            self.computeXRange()
            self.computeYRange()
            self.computeZRange()
            grapher.GUIGrapher.plot(self)
        else:
            self.clear()
        self.draw()
        self.update()
        
    configure=config

    def _configNoDraw(self,cnf=None,**kw):
        return grapher.GUIGrapher._configNoDraw(self,cnf,**kw)
    _configureNoDraw = _configNoDraw

    def __optionCallback(self,key,value,options):
        gc.collect()
        if key == "runner":
            self.cget("bifurcation_diagram").read(value.getBifurcation_diagram())
            self.cget("solution").read(value.getSolution())
        elif key == "bifurcation_diagram_filename":
            try:
                self.cget("bifurcation_diagram").readFilename(value)
            except IOError:
                pass
        elif key == "solution_filename":
            try:
                self.cget("solution").readFilename(value)
            except IOError:
                pass
            if self.cget("label") != [0]:
                self.__optionCallback("label",self.cget("label"),options)
        elif key == "solution":
            if self.cget("label") != [0]:
                self.__optionCallback("label",self.cget("label"),options)
        elif key == "label":
            labels = self.cget("solution").getLabels()
            if not isinstance(value, (list, tuple)):
                value = [value]
            if value in [["all"], ["All"]]:
                value = labels
            options["index"] = [j for v in value
                    for j, l in enumerate(labels) if l == v]
        elif key == "index":
            labels = self.cget("solution").getLabels()
            if not isinstance(value, (list, tuple)):
                value = [value]
            options["label"] = [labels[v] for v in value]
        elif key in ["bifurcation_x","bifurcation_y","bifurcation_z",
                     "solution_x","solution_y","solution_z"]:
            if not isinstance(value, (list, tuple)):
                value = [value]
        elif key == "letter_symbols":
            keys = ["bifurcation_symbol", "limit_point_symbol",
                    "hopf_symbol", "period_doubling_symbol","torus_symbol",
                    "user_point_symbol", "error_symbol",
                    "1_1_resonance_symbol", "1_2_resonance_symbol",
                    "1_3_resonance_symbol", "1_4_resonance_symbol",
                    "bogdanov_takens_symbol", "cusp_symbol",
                    "generalized_hopf_symbol", "zero_hopf_symbol",
                    "fold_flip_symbol", "fold_torus_symbol",
                    "flip_torus_symbol", "torus_torus_symbol"]
            if value:
                values = ["B", "L", "H", "D", "T", "U", "X",
                          "R1", "R2", "R3", "R4", "BT", "CP", "GH", "ZH",
                          "LPD", "LTR", "PTR", "TTR"]
            else:
                values = ["square", None,
                          "fillsquare", "diamond", "filldiamond",
                          None, None,
                          "filldiamond", "filldiamond",
                          "filldiamond", "filldiamond",
                          "circle", None,
                          "triangle", "doubletriangle",
                          "doubletriangle", "doubletriangle",
                          "doubletriangle", "doubletriangle"]
            options.update(dict(zip(keys, values)))

        # We only recreate the data if one of the above options gets set
        # We can't just recreate the data for any option since:
        #   1)  It is inefficient
        #   2)  The range gets recomputed when we create new data, so
        #       minx, miny, maxx, and maxy all never get used.
        self.__needsPlot = "yes"

    def _plotNoDraw(self):
        self.delAllData()
        ty = self.cget("type")
        columns = [self.cget(ty+"_x"),self.cget(ty+"_y"),self.cget(ty+"_z")]
        for coord in range(3):
            if (type(columns[coord]) != type([]) and
                type(columns[coord]) != type(())):
                columns[coord] = [columns[coord]]
        names, columns = self.__makeaxistitles(*columns)
        m = max([len(column) for column in columns])
        for coord, column in enumerate(columns):
            if column is not None and len(column) == 1:
                columns[coord] = column * m
        plot = True
        if ty == "bifurcation":
            sol = self.cget(ty+"_diagram")
        else:
            sol = self.cget(ty)
        if ty == "bifurcation" and len(sol) and len(sol[0]):
            self.__plot7(*columns)
        elif ty == "solution" and len(sol) > 0:
            self.__plot8(*columns)
        else:
            plot = False
        if plot:
            label = {}
            for coord in "x","y","z":
                label[coord] = self[coord+"label"]
                if self.config(coord+"label")[3] is None:
                    namescoord = names[{"x": 0, "y": 1, "z": 2}[coord]]
                    if namescoord is None:
                        label["z"] = None
                    else:
                        label[coord] = ", ".join(namescoord)
            grapher.GUIGrapher._configNoDraw(self,xlabel=label["x"],
                                             ylabel=label["y"],
                                             zlabel=label["z"])

    def plot(self):
        self._plotNoDraw()
        self.clear()
        self.computeXRange()
        self.computeYRange()
        self.computeZRange()
        grapher.GUIGrapher.plot(self)
        self.draw()

    def __makeaxistitles(self,xcolumns,ycolumns,zcolumns):
        # parse coordinate names from bifurcation diagram/solution
        # then construct the titles from these names
        ty = self.cget("type")
        indepvarname = None
        if ty == "bifurcation":
            solution = self.cget(ty+"_diagram")
            # use the "branches" member for parseB objects
            solution = getattr(solution,"branches",solution)
        else:
            solution = self.cget(ty)
        parsecoordnames = []
        # first go by column
        for s in solution:
            # check for any "MAX"-style member and replace, e.g., U(1) by
            # MAX U(1)
            if ty == "bifurcation":
                for i in range(min(len(parsecoordnames),len(s.coordnames))):
                    name = s.coordnames[i]
                    namelist = name.split(None,1)
                    if (len(namelist) > 1 and 
                        namelist[0] in ["MAX", "MIN", "INTEGRAL", "L2-NORM"] and
                        parsecoordnames[i] == namelist[1].strip()):
                        parsecoordnames[i] = name.strip()
            elif indepvarname is None:
                indepvarname = s.indepvarname
            if len(s.coordnames) > len(parsecoordnames):
                parsecoordnames.extend([name.strip() for name in
                                        s.coordnames[len(parsecoordnames):]])
        # then add anything not already there
        for s in solution:
            for name in s.coordnames:
                sname = name.strip()
                if sname not in parsecoordnames:
                    parsecoordnames.append(sname)
                                    
        # override coordnames with user/autorc provided ones
        if ty == "solution":
            if self.cget("solution_indepvarname"):
                indepvarname = self.cget("solution_indepvarname")
            elif indepvarname is None:
                indepvarname = "t"
        coordnames = self.cget(ty+"_coordnames") or []
        coordnames = coordnames[:]
        # but if that list is too short, extend it
        if len(coordnames) < len(parsecoordnames):
            coordnames.extend(parsecoordnames[len(coordnames):])
        self._coordnames = coordnames

        # construct translation dictionary for unames/parnames in autorc
        labelnames = self.cget("labelnames") or {}
        labelnames = labelnames.copy()
        for key in list(labelnames):
            for prefix in "MIN ", "MAX ", "INTEGRAL ", "L2-NORM ":
                if prefix+key not in labelnames:
                    labelnames[prefix+key] = prefix+labelnames[key]

        # construct titles
        names = [["Error"],["Error"]]
        lx = len(xcolumns)
        ly = len(ycolumns)
        lz = len(zcolumns)
        if lx == 1: lx = max(ly, lz)
        if ly == 1: ly = max(lx, lz)
        if lz == 1: lz = max(lx, ly)
        if (len(solution) > 0 and lx == ly == lz and 
            (ty == "solution" or len(solution[0]) > 0)):
            names = [[],[],[]]
            for j in range(3):
                columns = [xcolumns,ycolumns,zcolumns][j]
                for col in columns:
                    if col is None:
                        names[j] = None
                        continue
                    if type(col) == type(1):
                        # numerical column: check limits
                        if indepvarname is not None and col == -1:
                            col = indepvarname
                        else:
                            for s in solution:
                                if col < len(s.coordnames):
                                    col = coordnames[col]
                                    break
                            if type(col) == type(1):
                                print("The %s-coordinate (set to column %s) "
                                      "is out of range"%(["x","y","z"][j],col))
                                col = "Error"
                    elif col not in coordnames and (indepvarname is None or
                                                    col != indepvarname):
                        print("Unknown column name: %s"%(col))
                        col = "Error"
                    names[j].append(labelnames.get(col, col))

        # translate xcolumns/ycolumns to use parsed coordnames
        ucoordnames = self.cget(ty+"_coordnames") or []
        cols = [xcolumns,ycolumns,zcolumns]
        if ucoordnames != []:
            cols = []
            for columns in [xcolumns,ycolumns,zcolumns]:
                ncol = []
                for col in columns:
                    try:
                        indx = ucoordnames.index(col)
                        col = parsecoordnames[indx]
                    except (ValueError, TypeError):
                        pass
                    ncol.append(col)
                cols.append(ncol)
        return names,cols

    def __plot7branch(self,branch,xcolumns,ycolumns,zcolumns):
        symbollist = [
            [[1,6], "bifurcation_symbol"],
            [[2,5], "limit_point_symbol"],
            [[3],   "hopf_symbol"],
            [[-1],  "bogdanov_takens_symbol"],
            [[-2],  "cusp_symbol"],
            [[-32], "generalized_hopf_symbol"],
            [[-3],  "zero_hopf_symbol"],
            [[7],   "period_doubling_symbol"],
            [[8],   "torus_symbol"],
            [[-5],  "1_1_resonance_symbol"],
            [[-6],  "1_2_resonance_symbol"],
            [[-7],  "1_3_resonance_symbol"],
            [[-8],  "1_4_resonance_symbol"],
            [[23,83], "fold_torus_symbol"],
            [[77,87], "flip_torus_symbol"],
            [[28,78], "fold_flip_symbol"],
            [[88],  "torus_torus_symbol"],
            [[-4],  "user_point_symbol"]]
        specialsymbols = [-32,23,83,77,87,28,78,88]
        dp = self.cget("stability")
        coordnames = branch.coordnames
        for j in range(len(xcolumns)):
            labels = []
            for i,l in branch.labels.sortByIndex():
                label = None
                for k in l:
                    v = l[k]
                    if "LAB" in v:
                        label = v
                        break
                if label is None:
                    continue
                lab = label["LAB"]
                TYnumber = label["TY number"]
                if TYnumber not in specialsymbols:
                    if TYnumber>=0:
                        TYnumber=TYnumber%10
                    else:
                        TYnumber=-((-TYnumber)%10)
                text = ""
                if lab != 0:
                    text = str(lab)
                symbol = None
                for item in symbollist:
                    if TYnumber in item[0]:
                        symbol = item[1]
                if symbol is not None:
                    symbol = self.cget(symbol)
                elif TYnumber not in [0,4,9]:
                    symbol = self.cget("error_symbol")
                labels.append([i, text, symbol])
            xycols = []
            for col in [xcolumns[j],ycolumns[j],zcolumns[j]]:
                if col is None:
                    break
                if type(col) != type(1):
                    try:
                        col = coordnames.index(col)
                    except ValueError:
                        # check if we have an item that starts with
                        # MAX, MIN, INTEGRAL, or L2-NORM
                        # in that case also plot U(1) if given MAX U(1)
                        namelist = col.split(None,1)
                        if len(namelist) < 2 or (namelist[0] not in 
                                  ["MAX", "MIN", "INTEGRAL", "L2-NORM"]):
                            break
                        try:
                            col = coordnames.index(namelist[1])
                        except ValueError:
                            break
                try:
                    xy = branch.coordarray[col]
                except IndexError:
                    break
                xycols.append(xy)
            if len(xycols) < 2 or (zcolumns[j] is not None and len(xycols) < 3):
                continue
            [x,y] = xycols[:2]
            if len(xycols) == 3:
                z = xycols[2]
            else:
                z = None
            if self.cget("coloring_method") == "branch":
                color = abs(branch.BR)-1
            elif self.cget("coloring_method") == "type":
                color = branch.TY
            else:
                color = None
            if dp:
                #look at stability:
                newsect = 1
                old = 0
                stability = branch.stability()
                addedlabels = []
                for pt in stability:
                    abspt = abs(pt)
                    if abspt > 1 or pt == stability[-1]:
                        if z is None:
                            v = x[old:abspt],y[old:abspt]
                        else:
                            v = x[old:abspt],y[old:abspt],z[old:abspt]
                        self.addArrayNoDraw(v,newsect,color,stable=pt<0)
                        for label in labels:
                            if (old <= label[0] and label[0] < abspt and
                                label not in addedlabels):
                                self.addLabel(len(self)-1, label[0] - old,
                                              label[1], label[2])
                                addedlabels.append(label)
                        old = abspt - 1
                        newsect = 0
            else:
                if z is None:
                    self.addArrayNoDraw((x,y),1,color)
                else:
                    self.addArrayNoDraw((x,y,z),1,color)
                for label in labels:
                    self.addLabel(len(self)-1, *label)

    def __plot7(self,xcolumns,ycolumns,zcolumns):
        self.delAllData()
        solution = self.cget("bifurcation_diagram")
        branches = getattr(solution,"branches",solution)
        if len(xcolumns) == len(ycolumns) == len(zcolumns):
            for branch in branches:
                self.__plot7branch(branch,xcolumns,ycolumns,zcolumns)

    def __plot8solution(self,sol,index,xcolumns,ycolumns,zcolumns):
        indepvarname = sol.indepvarname
        tm = sol[indepvarname]
        label = sol["Label"]
        if self.cget("solution_indepvarname"):
            indepvarname = self.cget("solution_indepvarname")
        coordnames = sol.coordnames
        if self.cget("coloring_method") == "branch":
            color = sol["BR"]-1
        elif self.cget("coloring_method") == "type":
            color = sol["TY number"]
        else:
            color = None
        for j in range(len(xcolumns)):
            labels = []
            xycols = []
            for col in [xcolumns[j],ycolumns[j],zcolumns[j]]:
                if type(col) != type(1):
                    if indepvarname == col:
                        col = -1
                    else:
                        try:
                            col = coordnames.index(col)
                        except ValueError:
                            break
                if col == -1:
                    xy = tm
                else:
                    try:
                        xy = sol.coordarray[col]
                    except IndexError:
                        break
                xycols.append(xy)
            if len(xycols) < 2:
                continue
            [x,y] = xycols[:2]
            z = None
            if len(xycols) == 3:
                z = xycols[2]
            if not(self.cget("mark_t") is None):
                for i in range(len(tm)):
                    if i != 0 and tm[i-1] <= self.cget("mark_t") < tm[i]:
                        labels.append({"index": i,
                                       "text": "",
                                       "symbol": "fillcircle"})
            if len(tm) <= 15:
                index = 1
                if len(tm) <= 1:
                    index = 0
            labels.append({"index": index, "text": str(label), "symbol": ""})
            # Call the base class config
            if len(x) > 0:
                if z is None:
                    self.addArrayNoDraw((x,y),color=color)
                else:
                    self.addArrayNoDraw((x,y,z),color=color)
            for lab in labels:
                self.addLabel(len(self)-1, lab["index"], lab["text"],
                              lab["symbol"])

        index = index + 10
        if index > len(tm):
            index = 14
        return index

    def __plot8(self,xcolumns,ycolumns,zcolumns):
        self.delAllData()
        solution = self.cget("solution")
        if len(xcolumns) == len(ycolumns) == len(zcolumns):
            index = 9
            for ind in self.cget("index"):
                sol = solution.getIndex(ind)
                index = self.__plot8solution(sol,index,xcolumns,ycolumns,
                                             zcolumns)



def test():
    from auto import parseB
    from auto import parseS
    import sys
    foo = plotter(bifurcation_diagram_filename="../test_data/fort.7",
                  solution_filename="../test_data/fort.8")

    foo.plot()
    foo.pack()
    foo.update()
    print("Hit return to continue")
    raw_input()
    foo.clear()
    foo.update()
    print("Hit return to continue")
    raw_input()
    foo.clear()
    foo.config(type="solution",label=[6])
    foo.plot()
    foo.update()
    print("Hit return to continue")
    raw_input()
    foo.clear()
    foo.config(index=[3])
    foo.plot()
    foo.update()
    print("Hit return to continue")
    raw_input()

if __name__ == "__main__":
    test()








__author__ = "ak"
__version__ = "0.2.4"
import configparser
import os
import warnings

from pyfurc.core import (
    BifurcationProblem,
    BifurcationProblemSolution,
    BifurcationProblemSolver,
    Dof,
    Energy,
    Load,
    Parameter,
    PhysicalQuantity,
)
from pyfurc.tools import setup_auto_exec_env
from pyfurc.util import (
    AutoCodePrinter,
    AutoOutputReader,
    AutoParameters,
    DataDir,
    HiddenAutoParameters,
    ParamDict,
)



#!/usr/bin/env python
#    Visualization for Bifurcation Manifolds
#    Copyright (C) 1997 Randy Paffenroth and John Maddocks
#
#    This library is free software; you can redistribute it and/or
#    modify it under the terms of the GNU  General Public
#    License as published by the Free Software Foundation; either
#    version 2 of the License, or (at your option) any later version.
#
#    This library is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#    Library General Public License for more details.
#
#    You should have received a copy of the GNU Library General Public
#    License along with this library; if not, write to the Free
#    Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
#    MA 02111-1307, USA

import sys
from auto import AUTOExceptions
from auto import AUTOutil
from auto import parseB

# The parseC class parses an AUTO parameter file
# THESE EXPECT THE FILE TO HAVE VERY SPECIFIC FORMAT!
# it provides 4 methods:
# read and write take as an arguement either and input or output
#    stream (basically any object with has the method "readline"
#    for reading and "write" for writing)
#    
# readFilename and writeFilename take as an arguement a filename
#    in which to read/write the parameters (basically it opens the
#    file and then calles "read" or "write"
#    
# Once the data is read in the class provides a dictionary
# interface for manipulating the file.

class parseC(dict):

    line_comments = ["NDIM,IPS,IRS,ILP",
                     "NICP,(ICP(I),I=1 NICP)",
                     "NTST,NCOL,IAD,ISP,ISW,IPLT,NBC,NINT",
                     "NMX,RL0,RL1,A0,A1",
                     "NPR,MXBF,IID,ITMX,ITNW,NWTN,JAC",
                     "EPSL,EPSU,EPSS",
                     "DS,DSMIN,DSMAX,IADS",
                     "NTHL,(/,I,THL(I)),I=1,NTHL)",
                     "NTHU,(/,I,THU(I)),I=1,NTHU)",
                     "NUZR,(/,I,PAR(I)),I=1,NUZR)"]

    # These keys are preserved when reading in a new constants file,
    # and the keys are not in the constants file
    # Other keys are set to "None"
    special_keys = ["unames", "parnames", "e", "homcont"]

    def __init__(self, filename=None, **kw):
        self["new"] = True
        if filename is not None and type(filename) != type(""):
            dict.__init__(self, filename)
            self.update(**kw)
            return
        dict.__init__(self)
        for key in ['NPR', 'EPSS', 'ITMX', 'EPSU', 'ITNW', 'NBC',
            'IADS', 'IPS', 'IID', 'IIS', 'A1', 'DS', 'NMX', 'NTST',
            'NINT', 'NWTN', 'A0', 'EPSL', 'ISP', 'DSMIN', 'MXBF',
            'RL0', 'RL1', 'IPLT', 'ILP', 'NCOL',
            'DSMAX', 'ISW', 'IRS', 'IAD', 'JAC', 'NDIM', 'NPAR',
            'IBR', 'LAB', 'TY',
            'NUNSTAB', 'NSTAB', 'IEQUIB', 'ITWIST', 'ISTART',
            'sv', 's', 'dat',
            "THL","THU","UZR","UZSTOP","ICP","IREV","IFIXED","IPSI","U","PAR",
            "SP","STOP"]:
            self[key] = None
        if filename:
            self.readFilename(filename)
        self.update(**kw)

    def __setitem__(self, key, item):
        if key in ["THL", "THU", "UZR", "UZSTOP", "U", "PAR", "unames",
                   "parnames"]:
            if item is not None:
                if key == "PAR":
                    from auto import parseS
                    if isinstance(item,parseS.AUTOParameters):
                        item = [(k,item(k)) for k in range(1,len(item)+1)]
                if isinstance(item, dict):
                    item = item.items()
                new = []
                for x in item:
                    if isinstance(x, dict):
                        new.append([x["PAR index"], x["PAR value"]])
                    else:
                        new.append([x[0], x[1]])
                item = new
        elif key == "DS" and item == '-':
            if self[key] is None:
                item = -0.01
            else:
                item = -self[key]
        dict.__setitem__(self, key, item)

    def update(self, d=None, **kw):
        """     Change the options for this parseC object"""
        dct = d
        if dct is None:
            dct = {}
        dct.update(kw)
        for key in dct:
            value = dct[key]
            if self.get("homcont") is not None and key in self["homcont"]:
                self["homcont"][key] = value
            elif key in self or key in self.special_keys:
                self[key] = value
            elif key[:7] != 'Active ':
                raise AUTOExceptions.AUTORuntimeError(
                    "Unknown option: %s"%(key,))
            
    def setdefault(self, *args):
        k = args[0]
        if len(args) > 1:
            d = args[1]
        else:
            d = None
        if k not in self:
            self[k] = d
        return self.get(*args)
            
    def readFilename(self, filename):
        inputfile = open(filename, "r")
        self.read(inputfile)
        inputfile.close()

    def writeFilename(self, filename, new=False):
        output = open(filename, "w")
        self.write(output, new)
        output.close()

    def scanvalue(self, line, inputfile=None):
        # Scans line for a value
        # returns the value, followed by the rest of the line
        # The value is returned as a flat list of strings if the line 
        # starts with [ and otherwise as a single string
        level = 0
        quote = ' '
        quoteesc = False
        prev = ' '
        npos = 0
        value = ''
        isdict = False
        v = ''
        line = line.strip()
        i = 0
        while True:
            if i == len(line):
                if level == 0 or inputfile is None:
                    break
                line = inputfile.readline().strip()
                i = 0
            npos = i
            c = line[i]
            if quote == ' ':
                if c in [',', ' ']:
                    if level == 0:
                        break
                elif c == ':':
                    pass
                elif c in [']', '}']:
                    if c == '}':
                        isdict = False
                    if level == 1 and prev in ['[', '{']:
                        value = []
                    level = level - 1
                    if v != '':
                        value.append(v)
                    v = ''
                    if c == ']' and isdict:
                        value.append(']')
                else:
                    if prev in [',', ' ', ':'] and level > 0 and v != '':
                        value.append(v)
                        v = ''
                    if c in ['[','{']:
                        if level == 0:
                            value = []
                        level = level + 1
                        if c == '{':
                            isdict = True
                        elif isdict:
                            value.append('[')
                    elif c in ['"', "'"]:
                        quote = c
                    else:
                        v = v + c
            elif c == quote:
                # ignore "" and ''
                if (i+1 < len(line) and line[i+1] == c) or quoteesc:
                    quoteesc = not quoteesc
                else:
                    quote = ' '
            else:
                v = v + c
            prev = c
            i = i + 1
        if v != '':
            value = v
        while npos < len(line) and line[npos] in [" ", ","]:
            npos = npos + 1
        if npos >= len(line) - 1:
            line = ''
        else:
            line = line[npos:].strip()
        return value, line

    def parseline(self, line, userspec=False, inputfile=None, lineno=None):
        # parses a new-style constant file line and puts the keys
        # in the dictionary c; also used for the header of a b. file
        while line != "":
            line = line.strip()
            if line[0] in ['#', '!', '_']:
                return
            pos = line.find('=')
            if pos == -1:
                return
            key = line[:pos].strip()
            value, line = self.scanvalue(line[pos+1:],inputfile)
            if key in ['ICP', 'IREV', 'IFIXED', 'IPSI']:
                d = []
                for v in value:
                    try:
                        v = int(v)
                    except ValueError:
                        pass
                    d.append(v)
                value = d
            elif key in ['THU', 'THL', 'UZR', 'UZSTOP', 'U', 'PAR']:
                d = []
                i = 0
                while i < len(value):
                    try:
                        v0 = int(value[i])
                    except ValueError:
                        v0 = value[i]
                    if value[i+1] == '[':
                        i = i + 1
                        v1 = []
                        while i+1 < len(value) and value[i+1] != ']':
                            v1.append(parseB.AUTOatof(value[i+1]))
                            i = i + 1
                    else:
                        v1 = parseB.AUTOatof(value[i+1])
                    d.append([v0,v1])
                    i = i + 2
                value = self.__compactindexed(d)
            elif key in ['unames', 'parnames']:
                value = [[int(value[i]),value[i+1]] 
                         for i in range(0,len(value),2)]
            elif key in ['s', 'dat', 'sv', 'e', 'SP', 'STOP', 'TY']:
                pass
            elif key in self:
                if key[0] in 'IJKLMN':
                    if value[0] == '*':
                        value = 10**len(value)
                    elif key == 'IRS':
                        try:
                            value = int(value)
                        except ValueError:
                            pass
                    else:
                        value = int(value)
                else:
                    value=parseB.AUTOatof(value)
            elif lineno is not None:
                raise AUTOExceptions.AUTORuntimeError(
                    "Unknown AUTO constant %s on line %d"%(key,lineno))
            else:
                value = None
            if value is not None:
                if userspec:
                    self["Active "+key] = self[key]
                self[key] = value

    def read(self, inputfile):
        line = inputfile.readline()
        lineno = 1
        while line != '':
            data = line.split()
            if len(data) > 0:
                if data[0][0].isdigit():
                    break
                self.parseline(line,inputfile=inputfile,lineno=lineno)
            line = inputfile.readline()
            lineno += 1
                
        if line == '':
            return
        
        self["new"] = False
        self["NDIM"] = int(data[0])
        self["IPS"] = int(data[1])
        try:
            self["IRS"] = int(data[2])
        except ValueError:
            self["IRS"] = data[2]            
        self["ILP"] = int(data[3])

        line = inputfile.readline()
        data = line.split()
        icp = []
        for i in range(int(data[0])):
            d = data[i+1]
            try:
                d = int(d)
            except ValueError:
                pass
            icp.append(d)
        self["ICP"] = icp

        for line_comment in self.line_comments[2:7]:
            line = inputfile.readline()
            data = line.split()
            for d, key in zip(data, line_comment.split(",")):
                if key[0] in 'IJKLMN':
                    self[key] = int(d)
                else:
                    self[key] = parseB.AUTOatof(d)

        for key in ["THL", "THU", "UZR"]:
            line = inputfile.readline()
            data = line.split()
            item = []
            for i in range(int(data[0])):
                line = inputfile.readline()
                data = line.split()
                d = data[0]
                try:
                    d = int(d)
                except ValueError:
                    pass
                item.append([d, parseB.AUTOatof(data[1])])
            self[key] = item

    def __compactindexed(self, value):
        """compact THL/THU/UZR lists"""
        d = []
        v0s = []
        for v0, v1 in value:
            if v0 in v0s:
                if not AUTOutil.isiterable(v1):
                    v1 = [v1]
                # add to list when parameter was already encountered
                try:
                    d[v0s.index(v0)][1].extend(v1)
                except AttributeError:
                    d[v0s.index(v0)][1] = [d[v0s.index(v0)][1]]
                    d[v0s.index(v0)][1].extend(v1)
            else:
                v0s.append(v0)
                d.append([v0,v1])
        return d

    def __compactstr(self, value):
        """check if we can use more compact output than str..."""
        try:
            str1 = "%.5g" % value
            str2 = str(value)
            if float(str1) == float(str2) and 'e' in str1:
                return str1
            return str2
        except TypeError:
            l = []
            for v in value:
                l.append(self.__compactstr(v))
            return '['+", ".join(l)+']'
            
    def __newstr(self, lines=None):
        wdth3keys = ["NMX", "NPR", "NBC", "JAC", "e"]
        wdth5keys = ["EPSU", "EPSS"]
        if lines is None:
            lines = [
                ["e", "s", "dat", "sv"],
                ["unames", "parnames"],
                ["U", "PAR"],
                ["NDIM", "IPS", "IRS", "ILP"],
                ["ICP"],
                ["NTST", "NCOL", "IAD", "ISP", "ISW", "IPLT", "NBC", "NINT"],
                ["NMX", "NPR", "MXBF", "IID", "ITMX", "ITNW", "NWTN", "JAC"],
                ["EPSL", "EPSU", "EPSS"],
                ["DS", "DSMIN", "DSMAX", "IADS"],
                ["NPAR", "THL", "THU"],
                ["IIS", "IBR", "LAB", "TY"],
                ["UZR"],
                ["UZSTOP"],
                ["STOP"],
                ["SP"],
                ["RL0", "RL1", "A0", "A1"],
                ["NUNSTAB", "NSTAB", "IEQUIB", "ITWIST", "ISTART"],
                ["IREV", "IFIXED", "IPSI"]]
        olist = []
        for line in lines:
            pos = 0
            for key in line:
                value = self.get(key)
                if value is None:
                    continue
                if pos > 0:
                    olist.append(", ")
                pos = pos + 1
                if key in wdth3keys:
                    s = "%-3s=" % key
                elif key in wdth5keys:
                    s = "%-5s=" % key
                else:
                    s = "%-4s=" % key
                olist.append(s)
                if key in ["ICP", "IREV", "IFIXED", "IPSI", "STOP", "SP"]:
                    s = "  "+str(value)
                elif key in ["THL", "THU", "UZR", "UZSTOP", "U", "PAR"]:
                    l = ["'"+str(k)+"': "+self.__compactstr(v) for k, v in value]
                    s = "  {"+", ".join(l)+"}"
                elif key in ["unames", "parnames"]:
                    l = [str(k)+": '"+str(v)+"'" for k, v in value]
                    s = "{"+", ".join(l)+"}"
                elif key in ["sv", "s", "dat", "e", "TY"]:
                    value = "'"+str(value)+"'"
                    if key in wdth3keys:
                        s = "%5s" % value
                    else:
                        s = "%4s" % value
                elif key[0] in ["A", "D", "E", "R"]:
                    value = self.__compactstr(value)
                    s = "%6s" % value
                elif pos > 4:
                    s = "%2s" % value
                elif key in wdth3keys:
                    s = "%5s" % value
                else:
                    s = "%4s" % value
                olist.append(s)
            if pos > 0:
                olist.append("\n")
        return "".join(olist)

    def __oldstr(self):
        olist = [self.__newstr([
                ["e", "s", "dat", "sv"],
                ["unames", "parnames"],
                ["U", "PAR"],
                ["NPAR", "IBR", "LAB"],
                ["STOP"],
                ["SP"],
                ["NUNSTAB", "NSTAB", "IEQUIB", "ITWIST", "ISTART"],
                ["IREV", "IFIXED", "IPSI"]])]
            
        for j, line_comment in enumerate(self.line_comments):
            if j==1:
                s = " ".join([str(len(self["ICP"]))]+map(str,self["ICP"]))
            elif j==7:
                s = str(len(self["THL"]))
            elif j==8:
                s = str(len(self["THU"]))
            elif j==9:
                uzrlist = []
                for k, v in self["UZR"] or []:
                    if not AUTOutil.isiterable(v):
                        v = [v]
                    for vv in v:
                        uzrlist.append([k, vv])
                s = str(len(uzrlist))
            else:
                s = " ".join([str(self[d]) for d in line_comment.split(",")])
            olist.append(s+" "*(max(24-len(s),1))+line_comment+"\n")
            if j==7:
                for k, v in self["THL"] or []:
                    olist.append("%s %s\n" % (k, v))
            elif j==8:
                for k, v in self["THU"] or []:
                    olist.append("%s %s\n" % (k, v))
            elif j==9:
                for k, v in uzrlist:
                    olist.append("%s %s\n" % (k, v))

        return "".join(olist)

    def __str__(self):
        if self["new"]:
            return self.__newstr()
        else:
            return self.__oldstr()

    def write(self, output, new=False):
        if new and not self["new"]:
            for key in ["UZR", "THL", "THU"]:
                self[key] = self.__compactindexed(self[key])
            output.write(self.__newstr())
        else:
            output.write(str(self))

def pointtest(a):
    keys = ['NPR', 'UZR', 'EPSS', 'ITMX', 'EPSU', 'ITNW', 'NBC',
            'IADS', 'IPS', 'IID', 'A1', 'DS', 'NMX', 'NTST',
            'NINT', 'NWTN', 'A0', 'EPSL', 'ISP', 'DSMIN', 'MXBF',
            'RL0', 'RL1', 'ICP', 'IPLT', 'ILP', 'NCOL', 'THL',
            'DSMAX', 'ISW', 'IRS', 'THU', 'IAD', 'JAC', 'NDIM']
    for key in keys:
        if key not in a:
            raise AUTOExceptions.AUTORegressionError("No %s label"%(key,))

def test():
    print("Testing reading from a filename")
    foo = parseC()
    foo.readFilename("test_data/c.ab")    
    pointtest(foo)
    foo.write(sys.stdout, new=True)

    print("Testing reading from a stream")
    foo = parseC()
    fp = open("test_data/c.ab", "r")
    foo.read(fp)    
    pointtest(foo)

    print("parseC passed all tests")

if __name__ == '__main__' :
    test()





import pytest
import sympy as sp

import pyfurc as pf


@pytest.fixture()
def symmetric_bifurcation_problem():
    """Standard BifurcationProblem fixture for testing. Uses hinged cantilever, i.e. symmetric bifurcation with normalized parameters."""
    phi = pf.Dof("\\varphi")
    P = pf.Load("P")

    # first bifurcation point will be at exactly P = 1, phi = 0
    V = pf.Energy(1 / 2 * phi ** 2 - P * (1 - sp.cos(phi)))

    bf = pf.BifurcationProblem(V, name="hinged_cantilever")
    # Set maximum load to double the critical load
    bf.set_parameter("RL1", 2.0)

    return bf



import subprocess

from setuptools import setup
from setuptools.command.build_ext import build_ext
from setuptools.extension import Extension
from setuptools.command.install_lib import install_lib
from distutils.command.install_data import install_data
from distutils import log as distutils_logger
from wheel.bdist_wheel import bdist_wheel
import os
import shutil
from glob import glob


class custom_bdist_wheel(bdist_wheel):
    # def run(self):
    # subprocess.check_call(
    #     ["bash", "-x", "scripts/build-auto-07p.sh"]
    # )
    # super().run()

    def finalize_options(self):
        super().finalize_options()
        # Mark us as not a pure python package
        self.root_is_pure = False


class custom_install_data(install_data):
    def run(self):
        """
        Outfiles are the libraries that were built using cmake
        """
        self.outfiles = self.distribution.data_files


class custom_install_lib(install_lib):
    def run(self):
        self.skip_build = True
        include = []
        distutils_logger.info("Moving auto-07p files")
        # Copy library to wheel. Source dist contains the
        # whole ext/auto-07p directory via MANIFEST.in
        auto_lib_dir = os.path.join(self.distribution.bin_dir, "lib")
        auto_lib_file = os.path.join(auto_lib_dir, "libauto.so")
        install_auto_dir = os.path.join("pyfurc.ext", "auto-07p")
        # include AUTO license file
        license_file = os.path.join(
            self.distribution.bin_dir, "LICENSE"
        )
        license_target_dir = os.path.join(
            self.install_dir, install_auto_dir
        )
        os.makedirs(license_target_dir, exist_ok=True)
        shutil.move(license_file, license_target_dir)

        lib_target_dir = os.path.join(
            self.install_dir, install_auto_dir, "lib"
        )
        os.makedirs(lib_target_dir, exist_ok=True)
        shutil.move(auto_lib_file, lib_target_dir)

        self.distribution.data_files = [
            os.path.join(
                lib_target_dir, os.path.basename(auto_lib_file)
            ),
            os.path.join(
                license_target_dir, os.path.basename(license_file)
            ),
        ]
        self.distribution.run_command("install_data")

        super().run()


auto_ext = Extension(
    "auto-07p.ext",
    sources=[],
)


class custom_build_ext(build_ext):
    """
    Specialized builder for auto-07p
    """

    special_extension = auto_ext.name

    def log_subprocess_output(self, pipe, debug=False):
        for line in iter(pipe.readline, ""):  # b'\n'-separated lines
            if debug:
                distutils_logger.info(line.rstrip("\n"))
            else:
                distutils_logger.debug(line.rstrip("\n"))

    def build_extension(self, ext):
        if ext.name != self.special_extension:
            # Handle unspecial extensions with the parent class' method
            super().build_extension(ext)

        else:
            distutils_logger.info("Executing auto-07p configuration")
            auto_src_dir = os.path.join("ext", "auto-07p")
            # flags needed for building on a different machine than
            # the one running the code
            env = os.environ.copy()
            env["FFLAGS"] = "-Wall -fPIC"
            manylinux_build_tag = "x86_64-redhat-linux"
            target_build_tag = "x86_64-pc-linux-gnu"
            clean_cmd = [
                "make",
                "superclean"
            ]
            clean_process = subprocess.Popen(
                clean_cmd,
                cwd=auto_src_dir,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
            )
            with clean_process.stdout as stdout, clean_process.stderr as stderr:
                self.log_subprocess_output(stdout, debug=True)
                self.log_subprocess_output(stderr, debug=True)

            configure_cmd = [
                "./configure",
                "--enable-plaut=no",
                "--enable-plaut04=no",
                "--enable-plaut04-qt=no",
                "--enable-gui=no",
                "--without-openmp",
                "--without-mpi",
                f"--host={target_build_tag}",
            ]

            configure_process = subprocess.Popen(
                configure_cmd,
                cwd=auto_src_dir,
                env=env,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
            )
            with configure_process.stdout as stdout:
                self.log_subprocess_output(stdout, debug=True)

            distutils_logger.info("Building auto-07p")
            make_cmd = ["make", "-j3"]
            make_process = subprocess.Popen(
                make_cmd,
                cwd=auto_src_dir,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
            )
            with make_process.stdout as stdout:
                self.log_subprocess_output(stdout, debug=True)

            distutils_logger.info("Building auto-07p library")
            auto_lib_dir = os.path.join(auto_src_dir, "lib")
            build_lib_cmd = [
                "gfortran",
                "-v",
                "-shared",
                "-o",
                os.path.join(auto_lib_dir, "libauto.so"),
            ] + [f for f in glob(os.path.join(auto_lib_dir, "*.o"))]

            build_lib_process = subprocess.Popen(
                build_lib_cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                universal_newlines=True,
            )

            with build_lib_process.stdout as stdout:
                self.log_subprocess_output(stdout, debug=True)

            self.distribution.bin_dir = os.path.join(auto_src_dir)


setup(
    cmdclass={
        "bdist_wheel": custom_bdist_wheel,
        "install_lib": custom_install_lib,
        "install_data": custom_install_data,
        "build_ext": custom_build_ext,
    },
    ext_modules=[auto_ext],
)



#!/usr/bin/env autox
from auto import bifDiag, parseB, AUTOExceptions
from auto.graphics import windowPlotter
import code
import sys
try:
    import Tkinter
except ImportError:
    import tkinter as Tkinter
import select
try:
    import termios
except ImportError:
    pass
try:
    import readline
except ImportError:
    pass

class PyPlautInteractiveConsole(code.InteractiveConsole):
    def __init__(self,locals=None,b=None,s=None):
        code.InteractiveConsole.__init__(self,locals)
        if locals is None:
            return
        
        root=Tkinter.Tk()
        root.withdraw()
        self.handle = windowPlotter.WindowPlotter2D(root)
        self.handle.protocol("WM_DELETE_WINDOW", self.destroy)
        self.orig_destroy = self.handle.destroy
        self.handle.destroy = self.destroy
        try:
            self.tcattr = termios.tcgetattr(sys.stdin.fileno())
        except:
            pass

        if sys.platform == "cygwin":
            try:
                self.root = root
                readline.set_pre_input_hook(self.handleevents)
            except:
                pass

        #handle default options
        config = self.handle.configure()
        doptions = {"grid":       ["no", "no", "no", "yes", "yes"],
                    "use_labels": [   0,    1,    0,     1,     0],
                    "use_symbols":[   1,    1,    1,     1,     1],
                    "stability":  [   0,    1,    1,     1,     1]}
        dict = {}
        for i in range(5):
            if "d"+str(i) not in config:
                di = {}
                for k,v in doptions.items():
                    di[k] = v[i]
                dict["d"+str(i)] = (di,None)
        if "default_option" not in config:
            dict["default_option"] = ("d1",None)
        self.handle.grapher.addOptions(**dict)

        try:
            b = parseB.parseBR(b)
            bd = bifDiag.bifDiag(b,s,constants=b[0].c)
        except (IOError, AUTOExceptions.AUTORuntimeError):
            try:
                bd = bifDiag.bifDiag(b,s)
            except AUTOExceptions.AUTORuntimeError:
                sys.stderr.write(str(sys.exc_info()[1])+"\n")
                self.orig_destroy()
                sys.exit(1)
        dict = {
            "bifurcation_diagram": bd,
            "solution": bd(),
            "letter_symbols": False }
        for k,v in self[self["default_option"]].items():
            dict[k] = v
        for key in list(dict):
            # check if key was set in .autorc
            config = self.handle.config(key)
            if len(config) > 5 and config[5]:
                del dict[key]
        self.handle.config(**dict)
        self.defaults = {}
        for key in ["xlabel", "ylabel", "zlabel", "top_title"]:
            self.defaults[key] = self.handle.config(key)[3]
        self.plotdefaults = {}
        for key in ["top_title", "grid", "stability", "use_symbols",
                    "use_labels"]:
            self.plotdefaults[key] = self[key]
        self.normal_usage()
        self.b3d = False

    # this polling loop is here so that Cygwin Python does not "hang" the
    # plot window while Python waits for a user input
    def handleevents(self):
        while select.select([sys.stdin],[],[],0.02) == ([], [], []):
            self.root.dooneevent()

    def raw_input(self, prompt=None):
        line = ""
        while 1:
            if line == "":
                if self.b3d:
                    print(" ENTER <B3D> COMMAND\n")
                else:
                    print(" ENTER COMMAND\n")
                line = raw_input(prompt)
            line = self.process_input(line)

    def destroy(self):
        self.orig_destroy()
        try:
            termios.tcsetattr(sys.stdin.fileno(), termios.TCSADRAIN, self.tcattr)
        except:
            pass
        sys.exit()

    def process_input(self, line):
        lower = 0
        upper = 0
        its = 0
        for c in line:
            if c.islower():
                lower = 1
            if c.isupper():
                upper = 1
        if upper and not lower:
            line=line.lower()
        if line in ["stop","exit","quit","end"]:
            if self.b3d:
                self.b3d = False
                self["bifurcation_z"] = None
                return ""
            self.orig_destroy()
            sys.exit()
        elif line in ["scr","screen"]:
            return ""
        elif line in ["ss","set sym"]:
            return ""
        elif line == "sd":
            return ""
        elif line in ["lab","label"]:
            self.listlabels()
            return ""
        elif line in ["c","cl","clr","clear"]:
            self.handle.grapher.clear()
            self.handle.grapher.update()
            return ""
        elif line in ["save","sav","sa","s"]:
            self.savefile()
            return ""
        elif line == "help":
            return self.help()
        elif line in ["sy","sym","symbol"]:
            self["use_symbols"] = 1            
            return ""
        opts = []
        for j in range(6):
            if len(line) < j*2+2:
                break
            opts.append(line[j*2:j*2+2])
        if "sy" in opts:
            self["use_symbols"] = 1            
            its = 1
        if "dp" in opts:
            self["stability"] = 1
            its = 1
        if "ax" in opts:
            its = 1
            try:
                if self.b3d:
                    print(' ENTER THE NEW THREE AXES FOR (X,Y,Z) <B3D>')
                    line = raw_input().replace(","," ")
                    [xaxis,yaxis,zaxis] = map(int,line.split())
                    self["bifurcation_x"] = [xaxis-1]
                    self["bifurcation_y"] = [yaxis-1]
                    self["bifurcation_z"] = [zaxis-1]
                else:
                    print(' ENTER HORIZONTAL AND VERTICAL AXIS NUMBER (1,2,...) :')
                    line = raw_input().replace(","," ")
                    [xaxis,yaxis] = map(int,line.split())
                    self["bifurcation_x"] = [xaxis-1]
                    self["bifurcation_y"] = [yaxis-1]
            except:
                pass
        if "st" in opts:
            self.settitles(1,1,1,1)
            its = 1
        for i in range(5):
            dopt = "d"+str(i)
            if dopt in opts:
                self.dset = 1
                self.expert = 0
                self.handle.config(self[dopt])
                its = 1
        if "nu" in opts:
            its = 1
            self.handle.config(**self.defaults)
            self.handle.grapher.addRCOptions(self.defaults)
            self.normal_usage()
            self.handle.config(use_symbols=0,use_labels=0,
                               grid="no",stability=0)
            self.handle.grapher.plot()
        if "xp" in opts:
            its = 1
            self.expert = 1
        if "bd" in opts:
            index = opts.index("bd")
            if 2*index + 2 < len(line):
                if line[2*index+2] == "0":
                    self.plotbif(0)
                    return ""
            self.plotbif()
            return ""
        elif line[:2] == "2d":
            self.plotsol()
            return ""
        elif line[:2] == "3d":
            self.plotsol(use3d=True)
            return ""
        elif line == "b3d":
            self.b3d = True
            return ""
        elif line == "sda":
            return ""
        elif line == "sdo":
            return ""
        elif line == "sci":
            return ""
        elif line == "ssy":
            return ""
        elif line == "pa":
            return ""
        elif line == "sdd":
            return ""
        elif line == "sls":
            return ""
        elif line == "lda":
            return ""
        elif line == "us":
            return ""
        elif line == "lls":
            return ""
        elif line == "rss":
            return ""
        elif line == "rcs":
            return ""
        elif line == "res":
            return ""
        if its == 0:
            print(' ILLEGAL COMMAND - REENTER')
        return ""

    def normal_usage(self):
        self.dset = 1
        self.expert = 0
        self.icl = 0
        self.ict = 0
        for key in ["xlabel", "ylabel", "zlabel", "top_title"]:
            setattr(self, key, self.handle.config(key)[3])

    def savefile(self):
        print(' ENTER FILE NAME:')
        flname = raw_input().strip()
        if flname == '':
            flname = 'fig.1'
        self.handle.grapher.postscript(flname)
    
    def settitles(self,tit,rtit,axlb,raxlb):
        if self.b3d:
            labels = ["xlabel","ylabel","zlabel"]
        else:
            labels = ["xlabel","ylabel"]
        for key in labels:
            if axlb:
                if raxlb:
                    if not self.expert:
                        print (' ENTER '+ key[0].upper() +
                               ' AXIS LABEL BETWEEN THE QUOTES')
                    print(' "                              "')
                    setattr(self, key, raw_input().strip())
                label = getattr(self, key)
            else:
                label = ""
            if label is not None:
                self[key] = label
            self.handle.grapher.addRCOptions({key:label})
            if label is None:
                self.handle.grapher.plot()

        if rtit:
            if not self.expert:
                print(' ENTER TOP TITLE BETWEEN THE QUOTES')
            print(' "                                                            "')
            self.top_title = raw_input().strip()

            #if not self.expert:
            #    print ' ENTER BOTTOM TITLE BETWEEN THE QUOTES'
            #print ' "                                                            "'
        if tit:
            self["top_title"] = self.top_title
        else:
            self["top_title"] = ""
        self.handle.grapher.update()
        #ax.text(0.5,0,raw_input().strip(),transform=ax.transAxes)
        #self.handle.grapher.canvas.show()

    def listlabels(self):
        s = self["solution"]
        i = 0
        str='\n  THE LABELS ARE :    '
        for label in s.getLabels():
            str=str+"%5d"%(label)
            i=i+1
            if i==10:
                print(str)
                str='                      '
                i=0
        if len(str) > 22:
            print(str)

    def getopts(self):
        if self.expert:
            print(' LAB')
        else:
            print(' SOLUTION LABELS ?  ( <Y> OR <N> ) ')
        line = raw_input().strip()
        self["use_labels"] = line[0].lower() == 'y'
        if self.expert:
            print(' GL')
        else:
            print(' GRID LINES ?  (Y OR N)')
        line = raw_input().strip()
        if line[0].lower() == 'y':
            self["grid"] = "yes"
        else:
            self["grid"] = "no"
        tit = 0
        rtit = 0
        axlb = 0
        raxlb = 0
        if self.expert:
            print(' T - C')
            line = raw_input().strip()
            if len(line) > 0 and line[0].lower() == 'y':
                tit = 1
                if len(line) > 1 and line[1].lower() == 'y':
                    rtit   = 1
            print(' A - C')
            line = raw_input().strip()
            if len(line) > 0 and line[0].lower() == 'y':
                axlb = 1
                if len(line) > 1 and line[1].lower() == 'y':
                    raxlb = 1
        else:
            print(' TITLE ?  (Y OR N)')
            line = raw_input().strip()
            if len(line) > 0 and line[0].lower() == 'y':
                tit = 1
                if not self.ict:
                    self.ict = 1
                    rtit = 1
                else:
                    print(' CHANGE TITLES ?  (Y OR N)')
                    line = raw_input().strip()
                    if len(line) > 0 and line[0].lower() == 'y':
                        rtit = 1
            print(' AXES LABELS ?  (Y OR N)')
            line = raw_input().strip()
            if len(line) > 0 and line[0].lower() == 'y':
                axlb = 1
                if not self.icl:
                    self.icl = 1
                    raxlb  = 1
                else:
                    print(' CHANGE AXES LABELS ?  (Y OR N)')
                    line = raw_input().strip()
                    if len(line) > 0 and line[0].lower() == 'y':
                        raxlb = 1
        self.settitles(tit,rtit,axlb,raxlb)
        
    def enterlabels(self):
        self.listlabels()
        print('\n ENTER LABELS, OR <A> (ALL)\n')
        line = raw_input().lower()
        s = self["solution"]
        if line[0] == 'a':
            self["label"] = s.getLabels()
        else:
            line=line.replace(',',' ')
            line=line.replace(':','-')
            lablist = line.replace(',',' ').split()
            label = []
            for lab in lablist:
                try:
                    lrange = map(int,lab.split('-'))
                except:
                    print("    INVALID COMMAND,  REENTER")
                    return 0
                if len(lrange) == 2:
                    lrange = range(lrange[0],lrange[1]+1)
                label = label + lrange
            self["label"] = label
        return 1
        
    def plotsol(self,use3d=False):
        xaxsn = [1]
        yaxsn = [2]
        zaxsn = [3]
        
        self.handle.config(type = "solution")
        if use3d:
            self["solution_z"] = [1]
        if not self.enterlabels():
            return
        s = self["solution"]
        self.ndim=len(s[0]["data"][0]['u'])+1
        while 1:
            its = 0
            print('  NUMBER OF COMPONENTS :%5d'%(self.ndim))
            if use3d:
                axisstr = "%s %s %s"%(xaxsn,yaxsn,zaxsn)
            else:
                axisstr = "%s %s"%(xaxsn,yaxsn)
            axisstr = axisstr.replace("[","")
            axisstr = axisstr.replace("]","")
            axisstr = axisstr.replace(", ",",")
            print ('  ENTER AXES  (DEFAULT %s),'\
                   ' <D> (DISPLAY), OR <EX> (EXIT)'%axisstr)
            line = raw_input()
            lower = 0
            upper = 0
            for c in line:
                if c.islower():
                    lower = 1
                if c.isupper():
                    upper = 1
            if upper and not lower:
                line=line.lower()
            if line in ['end','ex','e']:
                return
            elif line == 'd':
                if not self.dset:
                    self.getopts()
            elif line in ["save","sav","sa","s"]:
                self.savefile()
            elif (line == '2d' and not use3d) or (line == '3d' and use3d):
                self.enterlabels()
            else:
                try:
                    axislist = map(int,line.replace(","," ").split())
                    if use3d:
                        xaxsn = axislist[:len(axislist)/3]
                        yaxsn = axislist[len(axislist)/3:len(axislist)/3*2]
                        zaxsn = axislist[len(axislist)/3*2:]
                    else:
                        xaxsn = axislist[:len(axislist)/2]
                        yaxsn = axislist[len(axislist)/2:]
                    xaxs = []
                    for x in xaxsn:
                        if x == 1:
                            xaxs.append("t")
                        else:
                            xaxs.append(x-2)
                    yaxs = []
                    for y in yaxsn:
                        if y == 1:
                            yaxs.append("t")
                        else:
                            yaxs.append(y-2)
                    self["solution_x"] = xaxs
                    self["solution_y"] = yaxs
                    self["solution_z"] = None
                    if use3d:
                        zaxs = []
                        for z in zaxsn:
                            if z == 1:
                                zaxs.append("t")
                            else:
                                zaxs.append(z-2)
                        self["solution_z"] = zaxs
                except:
                    print("    INVALID COMMAND,  REENTER")
                

    def plotbif(self,opt=None):
        if opt is None:
            while 1:
                if self.expert:
                    print(' LIMITS')
                elif self.b3d:
                    print('  ENTER XMIN,XMAX,YMIN,YMAX,ZMIN,ZMAX')
                else:
                    print('  ENTER XMIN,XMAX,YMIN,YMAX')
                line = raw_input().replace(","," ")
                try:
                    if self.b3d:
                        [xmin,xmax,ymin,ymax,
                         zmin,zmax] = map(float,line.split())
                    else:
                        [xmin,xmax,ymin,ymax] = map(float,line.split())
                    break
                except:
                    pass
            self.handle.config(type = "bifurcation")
            if self.b3d:
                self.handle.config(minx = xmin, maxx = xmax,
                                   miny = ymin, maxy = ymax,
                                   minz = zmin, maxz = zmax)
            else:
                self.handle.config(minx = xmin, maxx = xmax,
                                   miny = ymin, maxy = ymax)
        else:
            self.handle.config(type = "bifurcation")
            self.handle.grapher.plot()
        if not self.dset:
            self.getopts()
        
    def __setitem__(self,key,value):
        self.handle[key] = value

    def __getitem__(self,key):
        return self.handle[key]

    def help(self):
        print("""
         Principal PLAUT Commands :

  <BD0>   Bifurcation diagram with default limits
  <BD>    Bifurcation diagram with user-limits
  <AX>    To select bifurcation diagram axes
  <2D>    2D plot of labeled solutions
  <SAV>   To save the current plot in a file
  <CL>    To clear the graphics window
  <LAB>   List all labeled solutions in Unit 8
  <END>   To End PLAUT

        PLAUT Default Options :

  <D0>    Use solid curves, symbols, but no labels
  <D1>    As <D0>, showing stability and labels
  <D2>    As <D1>, without labels
  <D3>    As <D1>, with grid lines
  <D4>    AS <D2>, with grid lines

        Individual Options :

  <SY>    Use symbols for special points
  <DP>    Differential Plot (show stability)
  <ST>    Set up titles and axes labels
  <NU>    Normal usage (Reset special options)

  Press RETURN for more or Enter Command ...
  """)
        line = raw_input()
        if line != "":
            return line
        print("""
        Additional PLAUT Commands :

  <SCR>   To change the plot size
  <SS>    To define symbols
  <RSS>   Set symbol size
  <XP>    ""Expert"" (Abbreviated prompts)
  <SD>    To change curve type

  <SDA>   Set dash spacing
  <LDA>   Set dash size
  <SDO>   Set dot spacing
  <SCI>   Set circle spacing
  <SSY>   Set curve-symbol spacing
  <RCS>   Set curve-symbol size
  <SDD>   Set dash-dot spacing
  <SLS>   Set long-short dash spacing
  <LLS>   Set long-short dash size
  <PA>    Set plotting accuracy
  <RES>   Reset curves and symbols

  <3D>    3D plot of labeled solutions
  <B3D>   3D bifurcation diagram

         --- End of Help ---
  """)
        return ""

# Export the functions inside AUTOSimpleFunctions in a dictionary
# This also allows the setting of the log
def exportFunctions(log=None):
    PyPlautFunctionsInstance = PyPlautInteractiveConsole()
    dict = {}
    for name in PyPlautInteractiveConsole.__dict__:
        if name[0] != '_':
            dict[name] = getattr(PyPlautFunctionsInstance, name)
    return dict

# This is the Python syntax for making a script runable    
if __name__ == '__main__':
    sys.ps1=""
    if len(sys.argv) < 2:
        b='fort.7'
        s='fort.8'
    else:
        b='b.'+sys.argv[1]
        s='s.'+sys.argv[1]
    runner = PyPlautInteractiveConsole(exportFunctions(), b, s)
    runner.interact(" ENTER <HELP> IN CASE OF DIFFICULTY\n")



#!/usr/bin/env autox
#======================================================================
#======================================================================
#         Utility Program for Relabeling Labeled Solutions
#                        in AUTO Data Files
#======================================================================
#======================================================================

from auto import bifDiag
import sys

def relabel(n1b,n1s,n2b,n2s):
    bd = bifDiag.bifDiag(n1b,n1s)
    labels = bd().getLabels()
    l = 0
    for lab in bd.getLabels():
        lnum = 1
        if l > len(labels) or lab != labels[l]:
            k = 0
            for br in bd:
                lnum = lnum + len(br.headerlist)
                k = k + len(br.getLabels())
                if k > l:
                    break
                lnum = lnum + len(br)
            lnum = lnum + br(lab)["index"]
            sys.stdout.write("%s\n%s%d%s%d\n%s%d\n%s\n%s"%(
                    " WARNING : The two files have incompatible labels :",
                    "  b-file label ", lab, " at line ", lnum,
                    "  s-file label ", labels[l],
                    " New labels may be assigned incorrectly.",
                    " Continue ? : "))
            ch = raw_input()
            if ch != 'y' and ch != 'Y':
                print("Rewrite discontinued. Recovering original files")
                import os
                try:
                    os.remove(n2b)
                    os.remove(n2s)
                except:
                    pass
                return
        l = l + 1
    bd.uniquelyLabel()
    bd.writeFilename(n2b,n2s)

if __name__ == '__main__':
    if len(sys.argv) == 5:
        inb = sys.argv[1]
        ins = sys.argv[2]
        outb = sys.argv[3]
        outs = sys.argv[4]
    else:
        inb = "fort.27"
        ins = "fort.28"
        outb = "fort.37"
        outs = "fort.38"
    relabel(inb,ins,outb,outs)
